/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.*;
import java.awt.*;
import java.io.Serializable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import netscape.ldap.*;
import netscape.ldap.controls.*;
import netscape.ldap.util.*;

import util.*;

/**
 * DirNode is the node of each entry in a Directory tree.
 * VDirNode extends it in using VLV for virtual population.
 *
 * @author  rweltman
 * @version %I%, %G%
 */
public class VDirNode extends DirNode {
    /**
     * Default constructor
     */
    public VDirNode() {
        super();
    }

    /**
     * Constructs a bogus entry object in order to display some
     * information about why the object is bogus e.g. it's a
     * bad referral, the user does not have permission to see
     * it, etc.
     *
     * @param isBogus true if this object does not represent a
     * real entry
     */
    protected VDirNode( boolean isBogus ) {
        super( isBogus );
    }

    /**
     * Constructor of the entry object. It will not load the
     * directory information until later. This function will
     * just initialize the internal variables.
     *
     * @param model Directory Tree model.
     * @param dn DN of the Directory entry
     * @param displayName Name to use for rendering this node
     */
    public VDirNode( IDirModel model,
                     String dn,
                     String displayName ) {
        super( model, dn, displayName );
    }

    /**
     * Constructor of the entry object. It will not load the
     * directory information until later. This function will
     * just initialize the internal variables.
     *
     * @param model Directory Tree model.
     * @param dn DN of the Directory entry
     */
    public VDirNode( IDirModel model,
                     String dn ) {
        super( model, dn );
    }

    /**
     * Constructor of the entry object. It will not load the
     * directory information until later. This function will
     * just initialize the internal variables.
     *
     * @param dn DN of the Directory entry
     */
    public VDirNode( String dn ) {
        super( dn );
    }

    /**
     * Constructor of the entry object.
     *
     * @param model Directory Tree model.
     * @param entry LDAP entry.
     */
    public VDirNode( IDirModel model,
                     LDAPEntry entry ) {
        super( model, entry );
    }

    /**
     * Initialize the entry object with a name and icon.
     *
     * @param model Directory Tree model.
     * @param dn DN of the Directory entry
     * @param displayName Name to use for rendering this node
     */
    protected void initialize( IDirModel model,
                               String dn,
                               String displayName ) {
        super.initialize( model, dn, displayName );
        initDataModel();
    }

    /**
     * VLV support methods
     */
    static protected String getVLVSearchName( String dn ) {
        return "MCC " + dn.replace( ',', ' ' );
    }

    static protected String getVLVSearchDN( String dn ) {
        return "cn=" + getVLVSearchName( dn ) +
            ",cn=config,cn=ldbm";
    }

    /**
     * Return the DN of the entry describing a one-level search
     * index for a node
     *
     * @param dn The node which has or needs a VLV index
     * @return The DN of the entry describing the VLV index
     */
    static public String getVLVIndexDN( String dn ) {
        String indexName= "SN "+ getVLVSearchName( dn );
        return "cn=" + indexName + "," +
            getVLVSearchDN( dn );
    }

    protected boolean nodeHasIndex( String dn ) {
        String entryDN = getVLVIndexDN( dn );
        LDAPConnection ldc = getLDAPConnection();
        if ( ldc == null ) {
            return false;
        }
        try {
            LDAPEntry entry = ldc.read( entryDN );
            return true;
        } catch ( LDAPException e ) {
        }
        return false;
    }

    /**
     * VLV not used in this example to populate the tree
     */
    protected void initDataModel() {
        LDAPConnection ldc = getLDAPConnection();
        if ( ldc == null ) {
            Debug.println( "DirNode.initDataModel: " +
                           "no LDAP connection" );
            return;
        }
        if ( _dataModel == null ) {
            _dataModel =
                new VListEntryModel( ldc,
                                     getDN(),
                                     ldc.SCOPE_ONE,
                                     getChildFilter() );
            _dataModel.setReferralsEnabled(
                getModel().getReferralsEnabled() );
        }
    }

    // data model for child entries using VLV
    private VListEntryModel _dataModel = null;
}
