/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.Enumeration;
import java.io.UnsupportedEncodingException;
import netscape.ldap.*;

/**
 * Class for retrieving and storing preferences using LDAP. The
 * preferences are stored as a number of individual
 * entries under a user entry.
 *
 * @version 1.0
 * @author rweltman
 **/
public class LDAPPreferences3 extends Preferences {
    /**
     * Blank constructor for deserialization
     */
    public LDAPPreferences3() {
    }

    /**
     * Normal constructor specifying user ID or some such for
     * identifying the user's preferences, a URL identifying
     * where user entries for preferences are to be found,
     * and a password for the user
     *
     * @param userKey User ID or some other unique identifier
     * @param url LDAP URL indicating the host, port, and base DN
     * from where to search for the user
     * @param password Password for the user
     */
    public LDAPPreferences3( String url, String userKey, String password ) {
        super( userKey );
        try {
            setUrl( url );
        } catch ( java.net.MalformedURLException e ) {
        }
        _password = password;
    }

    /**
     * Set the host, port, and base DN through an LDAP URL
     *
     * @param url LDAP URL indicating the host, port, and base DN
     * from where to search for the user
     */
    public void setUrl( String url )
        throws java.net.MalformedURLException {
        _url = null;
        try {
            LDAPUrl inUrl = new LDAPUrl( url );
            // Make a complete LDAPUrl, including a search filter for
            // uid matching userKey
            String[] attrs = { "dn" }; // No attributes
            _searchUrl = new LDAPUrl( inUrl.getHost(), inUrl.getPort(),
                                      inUrl.getDN(), attrs,
                                      LDAPConnection.SCOPE_SUB,
                                      "uid=" + getUserKey() );
        } catch ( java.net.MalformedURLException e ) {
            System.err.println( "Incorrect URL syntax: " + url );
            throw e;
        }
    }

    /**
     * Report the host, port, and base DN through an LDAP URL
     *
     * @return LDAP URL indicating the host, port, and base DN
     * from where to search for the user
     */
    public String getUrl() {
        return (_url != null) ? _url.getUrl() : null;
    }

    /**
     * Save any changed preferences
     *
     * @return true on success
     */
    public boolean save() {
        try {
            Enumeration keys = getChanges().keys();
            // Any changes to save?
            if( !keys.hasMoreElements() ) {
                return true;
            }
            LDAPConnection ldc = new LDAPConnection();
            ldc.connect( _url.getHost(), _url.getPort() );
            ldc.authenticate( _url.getDN(), _password );
            String[] oclasses = { "top", "nsValueItem" };
            while( keys.hasMoreElements() ) {
                String name = (String)keys.nextElement();
                String value = "";
				try {
					value =
						new String( (byte[])getChanges().get( name ),
									"UTF8" );
				} catch ( UnsupportedEncodingException e ) {
				}
                // Create a DN for each preference
                String dn = "cn="+name + "," + _url.getDN();
                LDAPAttributeSet set = new LDAPAttributeSet();
                set.add( new LDAPAttribute( "objectclass",
                                            oclasses ) );
                // Save the value as a string
                set.add( new LDAPAttribute( "nsValueType",
                                            "nsValueCIS" ) );
                set.add( new LDAPAttribute( "nsValueCIS",
                                            value ) );
                set.add( new LDAPAttribute( "cn",
                                            name ) );
                LDAPEntry entry = new LDAPEntry( dn, set );
                System.out.println( "LDAPPreferences.save: " + entry );
                try {
                    ldc.add( entry );
                } catch ( LDAPException ex ) {
                    System.err.println( "LDAPPreferences.save: " + ex );
                    // If the entry already exists, replace it
                    if ( ex.getLDAPResultCode() ==
                         ex.ATTRIBUTE_OR_VALUE_EXISTS ) {
                        LDAPModification mod =
                            new LDAPModification(
                                LDAPModification.REPLACE,
                                new LDAPAttribute( "nsValueCIS",
                                                   value ) );
                        ldc.modify( dn, mod );
                    }
                }
            }
            getChanges().clear();
            return true;
        } catch ( LDAPException e ) {
            System.err.println( "LDAPPreferences.save: " + e );
        }
        return false;
    }

    /**
     * Read preferences from directory
     *
     * @return true if successful
     */
    protected boolean initialize() {
        clear();
        setInitialized( false );

        if ( _searchUrl == null ) {
            System.err.println( "Incorrect URL syntax" );
            return false;
        }

        // Search for a single matching entry
        if ( _url == null ) {
            String dn = searchEntry();
            if ( dn != null ) {
                // Found it
                String[] attrs = null; // Inefficient...
                // URL to read children of this entry
                _url = new LDAPUrl( _searchUrl.getHost(),
                                    _searchUrl.getPort(),
                                    dn, attrs,
                                    LDAPConnection.SCOPE_ONE,
                                    "objectclass=nsValueItem" );
                System.out.println( "URL: " + _url.getUrl() );
            }
        }
 
        // Now we know the entry containing the preferences
        boolean success = readChildren();
        if ( success ) {
            setInitialized( true );
        } else {
            clear();
        }

        return success;
    }

    /**
     * Get the preference values from an entry and store them
     * internally
     *
     * @param entry An LDAP entry containing preference attributes
     */
    protected void addPropertiesFromEntry( LDAPEntry entry ) {
        // Get the name of the attribute
        LDAPAttribute attr = entry.getAttribute( "cn" );
        String name = (String)attr.getStringValues().nextElement();
        Object value = null;
        // Get the type of the attribute
        String type = null;
        if ( (attr = entry.getAttribute( "nsValueType" )) != null ) {
            // Is the type specified?
            type = (String)attr.getStringValues().nextElement();
            if ( (attr = entry.getAttribute( type )) != null ) {
               value = attr.getByteValues().nextElement();
            }
        }
        // If type was not specified, look for all possible types
        for( int i = 0; (value == null) && (i < ATTRS.length); i++ ) {
            if ( (attr = entry.getAttribute( ATTRS[i] )) != null ) {
                value = attr.getByteValues().nextElement();
            }
        }
        if ( value != null ) {
            put( name, value );
        }

    }

    /**
     * Get the preference values from the directory and store
     * them internally
     *
     * @return true if the values could be read
     */
    protected boolean readChildren() {
        clear();
        LDAPSearchResults res = null;
        try {
            System.out.println( "LDAPPreferences.readChildren: reading " +
                                _url.getUrl() );
            res = LDAPConnection.search( _url );
        } catch ( LDAPException e ) {
            System.err.println( "LDAPPreferences.readChildren: " + e );
            return false;
        }
        while ( res.hasMoreElements() ) {
            try {
                LDAPEntry entry = res.next();
                addPropertiesFromEntry( entry );
                System.out.println( "LDAPPreferences.readChildren: found " +
                                    entry.getDN() );
            } catch ( LDAPException e ) {
                System.err.println( "LDAPPreferences.readChildren: " + e );
                continue;
            }
        }
        return true;
    }

    /**
     * Find the entry of this user
     *
     * @return the DN of the user entry, if found and unique,
     * otherwise null
     */
    protected String searchEntry() {
        clear();
        LDAPSearchResults res = null;
        try {
            System.out.println( "LDAPPreferences.searchEntry: " +
                                "searching " + _searchUrl.getUrl() );
            res = LDAPConnection.search( _searchUrl );
        } catch ( LDAPException e ) {
            System.err.println( "LDAPPreferences.searchEntry: " + e );
            return null;
        }
        String dn = null;
        while ( res.hasMoreElements() ) {
            if ( dn != null ) {
                System.err.println( "LDAPPreferences.searchEntry: " +
                                    "ambiguous user ID " + getUserKey() );
                clear();
                return null;
            }
            try {
                LDAPEntry entry = res.next();
                dn = entry.getDN();
                // Found a match
                System.out.println( "LDAPPreferences.searchEntry: found " +
                                    dn );
            } catch ( LDAPException e ) {
                System.err.println( "LDAPPreferences.searchEntry: " + e );
                continue;
            }
        }
        return dn;
    }

    public static void main( String[] args ) {
        Preferences prefs = new LDAPPreferences3(
            "ldap://localhost:389/o=airius.com",
            "dakers", "integument" );
        prefs.dump();
        prefs.setString( "teexfontsize", "12" );
        prefs.setString( "teextabsize", "4" );
        prefs.dump();
        prefs.save();
        System.exit( 0 );
    }

    // Possible attribute types for nsValueItem
    private final static String[] ATTRS = { "nsValueBin",
                                            "nsValueCIS",
                                            "nsValueCES",
                                            "nsValueInt",
                                            "nsValueDN",
                                            "nsValueTel" };

    private LDAPUrl _searchUrl = null; // Host, port, base DN
    private LDAPUrl _url = null; // Complete URL of user entry
    private String _password = null; // User password
}
