/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.*;
import java.io.Serializable;
import java.awt.event.KeyEvent;
import javax.swing.KeyStroke;
import javax.swing.tree.TreePath;
import javax.swing.event.*;
import netscape.ldap.*;

import util.*;

/**
  * Directory model is a tree model which describes how to
  * display the directory contents.
  *
  * @author  rweltman
  * @version %I%, %G%
  */
  
public class DirModel implements IDirModel,
                                 IDirContentListener,
                                 Serializable {

    /**
     * Default constructor for deserialization
     */
    public DirModel() {
        setChildFilter( initializeChildFilter() );
    }

    /**
     * Constructor of the model that doesn't populate the tree.
     * You must call initialize() to populate it.
     *
     * @param ldc Connection to LDAP server
     */
    public DirModel( LDAPConnection ldc ) {
        this();
        setLDAPConnection( ldc );
    }

    /**
     * Constructor of the model with the root object passed in.
     * Suffix nodes are retrieved and added to the tree.
     *
     * @param root root object
     * @param ldc Connection to LDAP server
     */
    public DirModel( Object root, LDAPConnection ldc ) {
        this( ldc );
        initialize( root );
    }

    /**
     * Set default filter which causes only container nodes to
     * be displayed in the tree
     */
    private String initializeChildFilter() {
        Hashtable containers = getContainers();
        /* Due to a server bug, "numsubordinates>=1" is not
         * indexed, but
         * "(&(numsubordinates=*)(numsubordinates>=1))" is,
         * in DS 4.0
         */
        String filter =
            "(|(&(numsubordinates=*)(numsubordinates>=1))";
        Enumeration e = containers.keys();
        while( e.hasMoreElements() )
            filter += "(objectclass=" +
                (String)e.nextElement() + ")";
        filter += ")";
        Debug.println( "DirModel.initializeChildFilter: " +
                       filter );
        return filter;
    }

    /**
     * Report if the model will supply node objects for tree
     * nodes. If false (the default), only container nodes will
     * appear.
     *
     * @return <CODE>true</CODE> if leaf nodes are to be
     * displayed in the tree
     */
    public boolean getAllowsLeafNodes() {
        return _allowLeafNodes;
    }

    /**
     * Determines if the model will supply node objects for
     * tree nodes. If false (the default), only container nodes
     * will appear.
     *
     * @param allow <CODE>true</CODE> if leaf nodes are to be
     * displayed in the tree
     */
    public void setAllowsLeafNodes( boolean allow ) {
        if ( allow ) {
            setChildFilter( "objectclass=*" );
        } else {
            setChildFilter( initializeChildFilter() );
        }
        _allowLeafNodes = allow;
        contentChanged();
    }

    /**
     * Used between DirNode and DirModel, to manage the search
     * filter used to find children of a node.
     *
     * @return The search filter to be used to find immediate
     * children
     */
    public String getChildFilter() {
        return _childFilter;
    }

    /**
     * Set the search filter used to find children of a node.
     *
     * @param filter The search filter to be used to find
     * direct children
     */
    public void setChildFilter( String filter ) {
        _childFilter = filter;
    }

    /**
     * Create root node and a node for each suffix
     *
     * @param root Root node for the tree. If null, a new root
     * node will be created and the root DSE will be searched
     * for suffixes
     */
    public void initialize( Object root ) {
        if ( root == null ) {
            root =
                new RootDirNode( this,
                                 getShowsPrivateSuffixes() );
            Debug.println(9, "DirModel.initialize: new root");
        } else {
            ((DirNode)root).setModel( this );
            Debug.println(9, "DirModel.initialize: old root=" +
                          root);
        }
        setRoot( root );
    }

    /**
     *  Get a child node of a node.
     *
     * @param node Parent node
     * @param index Position of the child
     * @return The child of the specified node
     */
    public Object getChild(Object node, int index) {
        IDirNode sn = (IDirNode) node;
        return sn.getChildAt(index);
    }

    /**
     *  return the number of children.
     * 
     * @param node node to be checked
     * @return number of children of the specified node.
     */
    public int getChildCount(Object node) {
        IDirNode sn = (IDirNode) node;
        return sn.getChildCount();
    }

    /**
      * Returns the index of a particular child node.
      * @param parent Parent node
      * @param child Child node
      * @return Position of the child
      */
    public int getIndexOfChild(Object parent, Object child) {
        return ((IDirNode) parent).getIndex(
                (IDirNode) child);
    }

    /**
     * Adds a listener that is interested in receiving
     * TreeModelListener events. Called by JTree.
     *
     * @param l An object interested in receiving
     * TreeModelListener events
     */
    public void addTreeModelListener(TreeModelListener l) {
        _listenerList.add(TreeModelListener.class, l);
    }

    /**
     * Removes a listener that is interested in receiving
     * TreeModelListener events. Called by JTree.
     *
     * @param l An object interested in receiving
     * TreeModelListener events
     */
    public void removeTreeModelListener(TreeModelListener l) {
        _listenerList.remove(TreeModelListener.class, l);
    }

    /**
      * Informs the tree that a particular node has changed
      *
      * @param node The node that changed
      * @see EventListenerList
      */
    public void fireTreeNodeChanged( DirNode node ) {
        // Guaranteed to return a non-null array
        Object[] listeners = _listenerList.getListenerList();
        TreeModelEvent e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for ( int i = listeners.length - 2; i >= 0; i -= 2 ) {
            if ( listeners[i] == TreeModelListener.class ) {
                // Lazily create the event:
                if (e == null) {
                    e = new TreeModelEvent(
                        this, node.getPath() );
                }
                TreeModelListener l =
                    (TreeModelListener)listeners[i + 1];
                l.treeNodesChanged(e);
            }
        }
    }

    /**
      * Informs tree that a particular node's structure has
      * changed and its view needs to be updated.
      * @param node The node at the root of the changes
      * @see EventListenerList
      */
    public void fireTreeStructureChanged( IDirNode node ) {
        // Guaranteed to return a non-null array
        Object[] listeners = _listenerList.getListenerList();
        TreeModelEvent e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for ( int i = listeners.length - 2; i >= 0; i -= 2 ) {
            if ( listeners[i] == TreeModelListener.class ) {
                // Lazily create the event:
                if (e == null)
                    e = new TreeModelEvent(
                        this, ((DirNode)node).getPath());
                TreeModelListener l =
                    (TreeModelListener)listeners[i + 1];
                l.treeStructureChanged( e );
            }
        }
    }

    /**
      * Called when user has altered the value for the item
      * identified by path to newValue.
      * Called by JTree
      *
      * @param path Path to the changed node
      * @param newValue New value of the node
      */
    public void valueForPathChanged(TreePath path,
                                    Object newValue) {
    }

    /**
     *  check whether the node is a leaf node or not
     *
     * @param node node to be checked
     * @return true if the node is leaf, false otherwise.
     */
    public boolean isLeaf( Object node ) {
        IDirNode sn = (IDirNode) node;
        return ( sn.isLeaf() );
    }

    /**
     * Called when the tree structure has changed radically;
     * read a new tree from the server.
     */
    public void contentChanged() {
        newTree();
    }

    void repaintObject( IDirNode node ) {
        Debug.println( "DirModel.repaintObject: " +
                       node );
        fireTreeStructureChanged( (DirNode)node );
    }

    private void newTree() {
        DirNode root =
            new RootDirNode( this, "",
                             getShowsPrivateSuffixes() );
        Debug.println(9, "DirModel.newTree: new root");
        setRoot( root );
        refreshTree();
    }

    private void refreshNode( IDirNode node ) {
        node.load();
        repaintObject( node );
    }

    private void refreshTree() {
        refreshNode( (IDirNode)getRoot() );
    }

    /**
     * Returns the server connection used to populate the tree.
     *
     * @return the server connection used to populate the tree
     */
    public LDAPConnection getLDAPConnection() {
        return _ldc;
    }

    /**
     * Sets the server connection used to populate the tree.
     *
     * @param ldc the server connection used to populate the
     * tree
     */
    public void setLDAPConnection( LDAPConnection ldc ) {
        _ldc = ldc;
    }

    /**
      * Returns root node of the tree.
      *
      * return Root node of the tree
      */
    public Object getRoot() {
        return _root;
    }

    /**
      * Sets root node of the tree.
      *
      * @param root Root node for the tree
      */
    public void setRoot(Object root) {
        _root = (IDirNode) root;
    }

    /**
     * Get the parameter which determines if the
     * ManagedSAIT control is sent with each search.
     *
     * @returns <CODE>true</CODE> if referrals are to be
     * followed
     */
    public boolean getReferralsEnabled() {
        return _followReferrals;
    }

    /**
     * Set a parameter for future searches, which determines if
     * the ManagedSAIT control is sent with each search. If
     * referrals are disabled, the control is sent and you will
     * receive the referring entry back.
     *
     * @param on <CODE>true</CODE> (the default) if referrals
     * are to be followed
     */
    public void setReferralsEnabled( boolean on ) {
        _followReferrals = on;
    }

    /**
     * Get the schema of the Directory instance
     *
     * @return A reference to a schema object.
     */
    public LDAPSchema getSchema() {
        if ( _schema == null ) {
            _schema = new LDAPSchema();
            try {
                _schema.fetchSchema( getLDAPConnection() );
            } catch ( LDAPException e ) {
                Debug.println( "DirModel.getSchema: " + e );
                _schema = null;
            }
        }
        return _schema;
    }

    /**
     * Sets a reference to the schema of the Directory instance
     *
     * @param schema A reference to a schema object.
     */
    public void setSchema( LDAPSchema schema ) {
        _schema = schema;
    }

    /**
     * Report if the model will show private suffixes.
     * If true (the default), private suffixes will appear.
     *
     * @return <CODE>true</CODE> if private suffixes are to be
     * displayed in the tree
     */
    public boolean getShowsPrivateSuffixes() {
        return _showPrivateSuffixes;
    }

    /**
     * Determines if the model will show private suffixes in
     * addition to public suffixes. If false (the default),
     * only public suffixes will appear.
     *
     * @param allow <CODE>true</CODE> if private suffixes are
     * to be displayed in the tree
     */
    public void setShowsPrivateSuffixes( boolean showPrivate ) {
        _showPrivateSuffixes = showPrivate;
        contentChanged();
    }

    /**
     * Get object classes which are to be considered containers
     * from a properties file.
     *
     */
    private static Hashtable initContainerNames() {
        Hashtable h = new Hashtable();
        String items =
            _resource.getString( _section, "containers" );
        Debug.println( "DirModel.initContainerNames" );
        if ( items != null ) {
            StringTokenizer st =
                new StringTokenizer( items, " " );
            int i = 0;
            while ( st.hasMoreTokens() ) {
                String name = st.nextToken().toLowerCase();
                Debug.println(
                    "  added container type " + name );
                h.put( name, name );
            }
        }
        return h;
    }

    /**
     * Add the name of an object class to be considered a
     * container.
     *
     * @param name Name of an object class to be considered a
     * container.
     */
    public void addContainerName( String name ) {
        _cContainers.put( name, name );
    }

    /**
     * Used between DirNode and DirModel, to manage the list of
     * objectclasses which are to be considered containers.
     *
     * @return A hashtable containing objectclasses to be
     * considered containers
     */
    public Hashtable getContainers() {
        if ( _cContainers == null ) {
            _cContainers = initContainerNames();
        }
        return _cContainers;
    }

    // Properties for this component (strings)
    static ResourceSet _resource =
        new ResourceSet( "dirtree" );
    // Section of the properties file to use
    private static final String _section = "EntryObject";

    // Active connection to directory
    private LDAPConnection _ldc;
    // Schema definitions
    private LDAPSchema _schema = null;
    // Control to use if referrals are not to be followed
    private static LDAPControl _manageDSAITControl =
            new LDAPControl( LDAPControl.MANAGEDSAIT, true, null );
    // Root node of the tree
    private IDirNode _root = null;
    private boolean _followReferrals = true;
    private boolean _allowLeafNodes = false;
    // List of possible container object classes
    private Hashtable _cContainers = null;
    // Filter string to search for immediate children
    private String _childFilter;
    // Helper object to manager event listeners
    protected EventListenerList _listenerList =
        new EventListenerList();
    // Set this to false to NOT show private suffixes
    private boolean _showPrivateSuffixes = true;
}
