/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

/**
 * This is a utility class to extract the reporting structure
 * from a directory subtree and create a tree of
 * DefaultMutableTreeNode entries. Each node contains a Hashtable
 * with various attribute values of the corresponding entry.
 *<P>
 * The following assumption is made about the directory
 * representation:<BR>
 * The entry of a user reporting to someone else has a <CODE>
 * manager</CODE> attribute, with a single value pointing to
 * another entry within the same subtree.<BR>
 */
import java.util.*;
import java.io.Serializable;
import javax.swing.tree.*;
import netscape.ldap.*;
import netscape.ldap.util.DN;

/**
 * ManagementParser
 * 
 * Utility class to create a tree of DefaultMutableTreeNode
 * objects representing the reporting relationships in a
 * directory
 *
 * @version 1.0
 * @author rweltman
 **/
public class ManagementParser implements Serializable {
    /**
     * Read person entries in a directory subtree, and
     * sort the results by reporting relationships
     *
     * @param ldc Connected and authenticated LDAPConnection
     * object
     * @param base Base DN from which to search for entries
     * @param rootDN DN of the top manager node to return
     * @return The root node of a tree, or <CODE>null</CODE>
     * on failure
     * @throws LDAPException on failure in searching
     */
    static public DefaultMutableTreeNode
    parseTree( LDAPConnection ldc,
               String base,
               String rootDN ) throws LDAPException {

        // Get root level managers and add them to the root;
        // keep references in a Hashtable. We consider a root
        // level manager to be anyone who doesn't have a
        // manager. In reality, some of these may be
        // highly placed individual contributors and some
        // may just not have their directory info up-to-date.
        DefaultMutableTreeNode root =
            new DefaultMutableTreeNode();
        Hashtable rootEntries =
            getRootLevelManagers( ldc, base, root );
        System.out.println( rootEntries.size() + " entries " +
                            "do not have a manager" );

        // Now get everyone who reports to someone. Some
        // of them are managers themselves.
        // Keep track of all managers in a Hashtable.
        Hashtable managerTable = new Hashtable();
        Hashtable allEntries =
            getSubordinates( ldc, base, managerTable );

        // Check how many root-level entries really are
        // managers
        int nRootManagers = 0;
        Enumeration en = rootEntries.keys();
        while( en.hasMoreElements() ) {
            String dn = (String)en.nextElement();
            if ( managerTable.containsKey( dn ) ) {
                nRootManagers++;
            }
        }
        System.out.println( nRootManagers + " top-level " +
                            "managers found" );

        System.out.println( allEntries.size() + " people " +
                            "who report to someone" );

        // Remove any invalid manager pointers
        int nInvalid = 0;
        en = managerTable.keys();
        while( en.hasMoreElements() ) {
            String dn = (String)en.nextElement();
            // If the manager DN doesn't exist in either the
            // root manager table or the subordinates table,
            // then we can't place any subordinates with that
            // manager.
            if ( !allEntries.containsKey( dn ) &&
                 !rootEntries.containsKey( dn ) ) {
//                 System.err.println( "No manager found: " +
//                                     dn + "; had these " +
//                                     "reports: " );
//                 Vector v = (Vector)managerTable.get( dn );
//                 for( int i = 0; i < v.size(); i++ ) {
//                     DefaultMutableTreeNode node =
//                         (DefaultMutableTreeNode)v.elementAt( i );
//                     System.out.println(
//                         "  " + getDNFromNode( node ) );
//                 }
                nInvalid++;
                managerTable.remove( dn );
            }
        }
        System.out.println( nInvalid + " managers no " +
                            "longer present" );
            
        // Create a cross-reference from DNs to nodes
        Hashtable managers = new Hashtable();
        // Add all subordinates to tree
        en = root.children();
        while( en.hasMoreElements() ) {
            DefaultMutableTreeNode node =
                (DefaultMutableTreeNode)en.nextElement();
            addEntriesToTree( node, managers,
                              managerTable );
        }

//        printManagers( managers );
        if ( rootDN != null ) {
            rootDN = normalizeDN( rootDN );
            Object o = managers.get( rootDN );
            return (DefaultMutableTreeNode)o;
        } else {
            return root;
        }
    }

    /**
     * Read person entries in a directory subtree, and
     * sort the results by reporting relationships
     *
     * @param ldc Connected and authenticated LDAPConnection
     * object
     * @param base Base DN from which to search for entries
     * @return The root node of a tree, or <CODE>null</CODE>
     * on failure
     * @throws LDAPException on failure in searching
     */
    static public DefaultMutableTreeNode
    parseTree( LDAPConnection ldc,
               String base ) throws LDAPException {
        return parseTree( ldc, base, null );
    }

    /**
     * Print a tree with indentation to indicate nesting
     *
     * @param root Top node to list
     */
    static public void
    printTree( DefaultMutableTreeNode root ) {
        // Iterate from the top of the tree down
        Enumeration en = root.depthFirstEnumeration();
        while( en.hasMoreElements() ) {
            DefaultMutableTreeNode node =
                (DefaultMutableTreeNode)en.nextElement();
            // Who is this node?
            Hashtable nodeEntry =
                (Hashtable)node.getUserObject();
            if ( nodeEntry == null ) {
                // root
                continue;
            }
            String dn = (String)nodeEntry.get( "dn" );
            for( int i = 0; i < node.getDepth(); i++ ) {
                System.out.print( '\t' );
            }
            System.out.println( dn );
        }
    }

    /**
     * Normalize a DN (spacing)
     *
     * @param dn String to be normalized
     * @return A normalized DN String
     */
    static protected String normalizeDN( String dn ) {
        return new DN( dn ).toString().toLowerCase();
    }

    /**
     * Create a tree node from info in an entry
     *
     * @param entry A directory entry with "cn" and optionally
     * also "title"
     * @return A tree node
     */
    static protected DefaultMutableTreeNode createEntryNode(
        LDAPEntry entry ) {
        String mgr = normalizeDN(
            getManagerFromEntry( entry ) );
        String name = getValueFromEntry( entry, "cn" );
        String title = getTitleFromEntry( entry );
        String dn = normalizeDN( entry.getDN() );
        Hashtable ht = new Hashtable();
        if ( name != null ) {
            ht.put( "name", name );
        }
        if ( title != null ) {
            ht.put( "title", title );
        }
        if ( mgr != null ) {
            ht.put( "manager", mgr );
        }
        ht.put( "dn", dn );
        return new DefaultMutableTreeNode( ht );
    }

    /**
     * Get all person entries that do not have a manager;
     * assume they are top-level managers or free entities
     * at the top, and add them to the root node.
     *
     * @param ldc Active connection to directory
     * @param base Base DN from which to do a subtree search
     * @param root Root node to add the entries found to
     * @return A Hashtable with all entries found
     */
    static protected Hashtable
    getRootLevelManagers( LDAPConnection ldc,
                          String base,
                          DefaultMutableTreeNode root )
        throws LDAPException {
        // Get all person entries that do not have a manager;
        // Assume they are top-level managers or free entities
        Hashtable rootEntries = new Hashtable();
        LDAPSearchConstraints cons =
            ldc.getSearchConstraints();
        cons.setMaxResults( 0 );
        LDAPSearchResults res =
            ldc.search( base,
                        ldc.SCOPE_SUB,
                        _rootFilter,
                        _fetchAttrs,
                        false,
                        cons );
        while ( res.hasMoreElements() ) {
            try {
                LDAPEntry entry = res.next();
                // Add this top level manager
//                 System.out.println( "Found root manager: " +
//                                     entry.getDN() );
                DefaultMutableTreeNode node =
                    createEntryNode( entry );
                root.add( node );
                rootEntries.put(
                    getDNFromNode( node ), node );
            } catch ( LDAPReferralException ref ) {
                // Just ignore referrals
            }
        }
        return rootEntries;
    }

    /**
     * Get all entries that have a manager. Keep track of all
     * referenced managers.
     *
     * @param ldc Active connection to directory
     * @param base Base DN from which to do a subtree search
     * @param managerTable Hashtable to store DNs and
     * nodes of managers
     * @return A Hashtable mapping all employee DNs to nodes
     */
    static protected Hashtable
    getSubordinates( LDAPConnection ldc,
                     String base,
                     Hashtable managerTable )
        throws LDAPException {
        LDAPSearchConstraints cons =
            ldc.getSearchConstraints();
        cons.setMaxResults( 0 );
        Hashtable allEntries = new Hashtable();
        // Find all person entries with a manager
        // Retrieve each one's name, title, and manager
        LDAPSearchResults res =
            ldc.search( base,
                        ldc.SCOPE_SUB,
                        _subordinateFilter,
                        _fetchAttrs,
                        false,
                        cons );
        int numReadEntries = 0;
        while ( res.hasMoreElements() ) {
            try {
                // Found an entry. Normalize the DN and use it
                // as the key in the allEntries Hashtable.
                LDAPEntry entry = res.next();
                String dn = normalizeDN( entry.getDN() );
                DefaultMutableTreeNode node =
                    createEntryNode( entry );
                allEntries.put( dn, node );
                // Get the manager DN and normalize it
                String managerDN =
                    getValueFromNode( node, "manager" );
                // Keep track of all managers' direct reports
                Vector v =
                    (Vector)managerTable.get( managerDN );
                if ( v == null ) {
                    v = new Vector();
                }
                v.addElement( node );
                managerTable.put( managerDN, v );
            } catch ( LDAPReferralException ref ) {
                // Just ignore referrals
            }
        }
        return allEntries;
    }

    /**
     * Get a single String attribute value from an entry
     *
     * @param entry A directory entry
     * @param name Name of the attribute
     * @return The attribute value, or <CODE>null</CODE> if
     * the attribute is not present
     */
    static protected String getValueFromEntry(
        LDAPEntry entry,
        String name ) {
        // Get the value from the entry
        LDAPAttribute attr = entry.getAttribute( name );
        if ( attr != null ) {
            Enumeration en = attr.getStringValues();
            if ( (en != null) && en.hasMoreElements() ) {
                return (String)en.nextElement();
            }
        }
        return null;
    }

    /**
     * Get a single String attribute value from a node
     *
     * @param node A tree node
     * @param name Name of the attribute
     * @return The attribute value, or <CODE>null</CODE> if
     * the attribute is not present
     */
    static protected String getValueFromNode(
        DefaultMutableTreeNode node,
        String name ) {
        Hashtable nodeEntry =
            (Hashtable)node.getUserObject();
        if( nodeEntry == null ) {
            return null;
        }
        return (String)nodeEntry.get( name );
    }

    /**
     * Get the DN from a node
     *
     * @param node A tree node
     * @return The DN value, or <CODE>null</CODE> if
     * the node doesn't have values
     */
    static protected String
    getDNFromNode( DefaultMutableTreeNode node ) {
        return getValueFromNode( node, "dn" );
    }

    /**
     * Get the manager DN from an entry
     *
     * @param entry A directory entry
     * @return The manager DN value, or <CODE>null</CODE> if
     * the manager attribute is not present
     */
    static protected String
    getManagerFromEntry( LDAPEntry entry ) {
        // Get the manager DN from the entry
        return getValueFromEntry( entry, "manager" );
    }

    /**
     * Get a title value from an entry
     *
     * @param entry A directory entry
     * @return String to use for title
     */
    static protected String
    getTitleFromEntry( LDAPEntry entry ) {
        for( int i = 0; i < _titleAttrs.length; i++ ) {
            String val =
                getValueFromEntry( entry, _titleAttrs[i] );
            if ( val != null ) {
                return val;
            }
        }
        return _defaultTitle;
    }

    /**
     * Add all entries in a Hashtable to a tree, sorting by
     * manager
     *
     * @param root The root node of the tree to add to. If it
     * is <CODE>null</CODE> it is allocated in the method.
     * @param managers A Hashtable which on return maps DNs
     * to tree nodes
     * @param managerTable A Hashtable of managers, where the
     * key is the DN and the value is a Vector of direct reports
     * corresponding to top-level managers
     * @return The root node of the tree
     */
    static protected DefaultMutableTreeNode addEntriesToTree(
        DefaultMutableTreeNode root,
        Hashtable managers,
        Hashtable managerTable ) {

        int childLess = 0;
        int childRich = 0;
        // Get all entries that have this manager
        String dn = getDNFromNode( root );
        Vector v = (Vector)managerTable.get( dn );
        if ( v != null ) {
            Enumeration children = v.elements();
            while( children.hasMoreElements() ) {
                DefaultMutableTreeNode nextNode =
                    (DefaultMutableTreeNode)
                    children.nextElement();
                String childDN = getDNFromNode( nextNode );
                // Recurse if this node is a manager
                if ( managerTable.containsKey( childDN ) ) {
                    childRich++;
                    addEntriesToTree( nextNode,
                                      managers,
                                      managerTable );
                } else {
                    childLess++;
                }
                // Add this child to the tree
                root.add( nextNode );
            }
            // Update manager Hashtable to point to this
            // node for the DN
            managers.put( dn, root );
        }
//         System.out.println( dn + " has " + childRich +
//                             " managers and " + childLess +
//                             " non-managers reporting" );
        return root;
    }

    /**
     * Print all manager entries
     *
     * @param managers Hashtable of managers
     */
    static protected void printManagers(
        Hashtable managers ) {
        System.err.println( "The following are managers: " );
        Enumeration names = managers.keys();
        while( names.hasMoreElements() ) {
            System.err.println( "\t<" +
                                (String)names.nextElement() +
                                ">" );
        }
    }

    /**
     * Create a tree of reporting relationships and print it
     *
     * @param args
     * <PRE>
     * Usage: ManagementParser -b BASEDN
     *                   [-D AUTHDN] [-w AUTHPASSWORD]
     *                   [-r ROOTMANAGERDN] [-h HOST] [-p PORT]
     *</PRE>
     */
    static public void main ( String[] args ) {
        // Parse arguments
        String host = "localhost";
        int port = 389;
        String base = null;
        String rootDN = null;
        String authDN = null;
        String authPassword = null;
        boolean invalid = false;
        int i = 0;
        while ( i < args.length ) {
            if ( args[i].compareTo("-b") == 0 ) {
                if ( (i + 1) < args.length ) {
                    base = args[i+1];
                    i++;
                }
            } else if (args[i].compareTo("-D" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authDN = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-w" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authPassword = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-h" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    host = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-p" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    port = Integer.parseInt(args[i+1]);
                    i++;
                }
            } else if ( args[i].compareTo("-r" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    rootDN = args[i+1];
                    i++;
                }
            } else {
                invalid = true;
            }
            i++;
        }

        if ( invalid || (base == null) ) {
            System.err.println(
                "Usage: ManagementParser -b BASEDN " +
                "[-D AUTHDN] [-w AUTHPASSWORD] " +
                "[-r ROOTMANAGERDN] [-h HOST] [-p PORT]" );
            System.exit( 1 );
        }

        // Connect to server
        LDAPConnection ldc = new LDAPConnection();
        try {
            ldc.connect( host,
                         port,
                         authDN,
                         authPassword );
        } catch ( LDAPException e ) {
            System.err.println( e );
            System.exit( 1 );
        }

        // Get a tree of users, organized by reporting
        // structure
        DefaultMutableTreeNode root = null;
        try {
            root = ManagementParser.parseTree( ldc, base,
                                               rootDN );
        } catch ( LDAPException e ) {
            System.err.println( e );
            System.exit( 1 );
        }
        ManagementParser.printTree( root );
    }

    // Filter string to retrieve users not reporting to anyone
    private static String _rootFilter =
        "(&(objectclass=person)" +
          "(!objectclass=nscphidethis)" +
          "(!(manager=*)))";
    // Filter to retrieve users reporting to someone
    private static String _subordinateFilter =
        "(&(objectclass=person)" +
          "(!objectclass=nscphidethis)" +
          "(manager=*))";
    // Attributes to check for a title
    private static String[] _titleAttrs =
        { "title", "description", "ou" };
    // Attributes to retrieve on search
    private static String[] _fetchAttrs =
        { "cn", "title", "manager", "description", "ou" };
    // Title to use if none present in entry
    private static String _defaultTitle = "Unknown";
}
