/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import netscape.ldap.*;
import netscape.ldap.util.*;
import netscape.ldap.controls.*;


/**
 * Phonebook servlet
 * 
 *
 * @version 1.0
 * @author tdahbura
 **/
public class Phonebook extends HttpServlet {

    // The following values are defaults.  They are
    // all settable using the properties file.
    // --------------------------------------------
    protected static String host = "localhost";  
    protected static int    port = 389;
    protected static String srchRoot = "o=airius.com"; 
    static String orgName = "Airius Communications";
    static String ownMail = "webmaster@airius.com";
    static int maxPhotoSize = 50000;
    String imagesURL = "/phonebook/images/";

    //The following values may all be customized for look/feel
    // Colors for listings lines
    final static String[] LIST_COLOR = {"#00CCCC","#009999"};
    // Colors for initial search page
    final static String srchPageColors =
        "\"#999999\" text=\"#000000\""+
        " link=\"#0000FF\" vlink=\"#FF00FF\""+
        " alink=\"#FF0000\"";
    // Colors for output of search
    final static String srchOutColors =
        "\"#999999\" text=\"#000000\""+
        " link=\"#0000FF\" vlink=\"#FF00FF\""+
        " alink=\"#FF0000\"";
    // End of customizable values ----------------------------

    
    protected static int scope = LDAPConnection.SCOPE_SUB; 
    private LDAPConnectionPool ldapPool;
    private LDAPCache readCache = null;

    final static String certAttr = "userCertificate;binary";
    final static String photoAttr = "jpegPhoto";
    // Window dimensions for photos
    final static int photoWinHeight = 100;
    final static int photoWinWidth = 100;
    final static String[] intAttrs = {"cn","telephonenumber",
                                      "mail",
                                      certAttr,photoAttr,
                                      "givenname","sn"};

    final static String[] extAttrs = {"cn","telephonenumber",
                                      "mail",
                                      certAttr,photoAttr,
                                      "givenname","sn"};

    private String hostIP = "127.0.0.1";

    final static String ACTION_FIELD = "cmd";
    final static String ACTION_SEARCH = "search";
    final static String ACTION_DETAIL = "getdetail";
    final static String ACTION_GETCERT = "getcert";
    final static String ACTION_GETPHOTO = "getphoto";
    final static String ACTION_EDIT = "editrecord";
    final static String ACTION_DOEDIT = "editsubmit";
    final static String ACTION_PWD = "editpwd";
    final static String ACTION_DOPWD = "pwdsubmit";

    final static String FLDNAME_SRCHKIND = "searchkind";
    final static String FLDNAME_SRCHSTRING = "search_string";
    final static String FLDNAME_DN = "dn";
    final static String FLDNAME_OLDPW = "oldpwd";
    final static String FLDNAME_NEWPW1 = "newpwd1";
    final static String FLDNAME_NEWPW2 = "newpwd2";
    final static String FLDNAME_NEWPHOTO = "newphoto";

    /**
      * return some descriptive information about us
      *
      * @return descriptive information
      */
    public String getServletInfo() {
        return "A servlet that handles phonebook lookup " +
            "using LDAP";
   }

    /**
     * Initialize the servlet, This is called once at startup.
     * Properties are read and prepared for future operations.
     * The LDAPConnectionPool and the LDAPCache instances are
     * prepared. The host IP address is stored for later use
     * by the inside/outside firewall checks.
      *
      * @param sc The servlet configuration
      * @exception ServletException on
      * failure to read properties or initialization of pool             
      */
    public void init( ServletConfig sc ) 
        throws ServletException {

        super.init(sc);

        // Read properties from the properties file.
        // If not found then create a placeholder file.
        Properties props = new Properties();
        File f = new File("phonebook.properties");
        if ( !f.exists() ) {
            System.out.println(f.getPath() + " not found");
            try {
                FileWriter fwout = new FileWriter(f);
                fwout.write("#Store your phonebook properties here");
                fwout.close();
                throw new ServletException
                    ("Cannot locate properties in ->"+f.getPath());
            } catch (IOException ie) {
                System.out.println("Cannot write properties file");
                throw new ServletException("Can't write properties file");
            }   
        }

        try {
            InputStream is = new BufferedInputStream(
                                new FileInputStream(f));
            props.load(is);
            is.close();
        } catch (IOException e) {
            throw new ServletException
                ("Cannot load properties in ->"+f.getPath());
        }

        // Read and save properties
        host = props.getProperty("host",host);
        try {
            port = Integer.parseInt(
                props.getProperty("port",String.valueOf(port)));
        } catch ( Exception ex ) {
            System.out.println("Invalid port");
            throw new ServletException(
                "Invalid port in properties file");
        }
        srchRoot = props.getProperty("srchroot",srchRoot);
        orgName = props.getProperty("orgname",orgName);
        ownMail = props.getProperty("ownmail",ownMail);
        imagesURL = props.getProperty("imgurl",imagesURL);
        int initPool=10;
        int maxPool=20;
        String authdn="";
        String authpw="";
        int cacheSize=25000;
        int cacheTime=3600;

        try {
            initPool = Integer.parseInt(
                props.getProperty("initpool","10"));
            maxPool = Integer.parseInt(
                props.getProperty("maxpool","20"));
            authdn = props.getProperty("authdn","");
            authpw = props.getProperty("authpw","");
            cacheSize = Integer.parseInt(
                props.getProperty("cachesize","250000"));
            cacheTime = Integer.parseInt(
                props.getProperty("cachetime","3600"));
            maxPhotoSize = Integer.parseInt(
                props.getProperty("maxphotosize","50000"));
        } catch (NumberFormatException ne) {
            throw new ServletException(
                "Invalid integer value in properties file");
        }

        try {
            // Create connection pool
            ldapPool = new LDAPConnectionPool(
                initPool,maxPool,host,port,authdn,authpw);
            if ( cacheSize > 0 ) {
                readCache = new LDAPCache(cacheTime,cacheSize);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new ServletException
                ("Unable to initialize LDAP connection pool");
        }

        // Get our IP Address and trim last octet
        try {
            hostIP = 
                java.net.InetAddress.getLocalHost().
                getHostAddress();
        } catch (java.net.UnknownHostException he) {
            hostIP = "127.0.0.1";
        }
        System.out.println("Servlet server IP="+hostIP);
        int i = hostIP.lastIndexOf( '.' );
        hostIP = hostIP.substring( 0, i );
    }

    /**
     * Destroy the servlet - called during an unload/shutdown
     */
    public void destroy() {
        if ( ldapPool != null ) {
            ldapPool.destroy();  // Release all connections
        }
        super.destroy();
    }
    
    /**
     * Handle HTTP Posts.
     * Check size of request and pass the request off to the
     * doGet handler if not too large.<BR>
     * If it is too large respond with an HTML error page.<BR>
     *
     * @param req The HTTP request
     * @param res The HTTP response
     *
     * @exception ServletException<BR>
     * @exception IOException<br>
     *               
     */
    public void doPost( HttpServletRequest req,
                        HttpServletResponse res )
        throws ServletException, IOException {
        // value chosen to limit denial of service attacks
        int limit = maxPhotoSize+1*1024;
        if ( maxPhotoSize==0 ) {
            limit = 20*1024;  //still within reason?
        }
        if (req.getContentLength() > limit) {  
            res.setContentType("text/html");
            PrintWriter out = res.getWriter();
            out.println("<HTML><HEAD><TITLE>Too big</TITLE></HEAD>");
            out.println("<BODY><H1>Error - content length &gt;"+
                limit+"k not allowed.");
            out.println("</H1></BODY></HTML>");
        } else {
            doGet(req, res);
        }
    }
   
    /**
      * Handles HTTP Get<BR>
        * Main logic processing for our servlet.  We look at the
        * arguments (form elements or url elements) and dispatch
        * to the servlet methods to handle these.<BR>
      *
      * @param req the HTTP request
        * @param res the HTTP response
        * <BR>
        * Exceptions can be thrown by servlet methods:
        * @exception ServletException<BR>
        * @exception IOException<br>
        *               
      */ 
    public void doGet( HttpServletRequest req,
                       HttpServletResponse res )
        throws ServletException, IOException {

        PrintWriter out = null;
        boolean isInternal = checkInside(req);
        String cmd = readFormData(req,ACTION_FIELD);

        // Set output content type to text/html; Change later
        // if necessary
        res.setContentType("text/html");

        // All routines output html except cert and photo
        if ( (!cmd.equals(ACTION_GETCERT)) &&
             (!cmd.equals(ACTION_GETPHOTO)) ) {
            out = res.getWriter();
        }

        //dispatch to the proper handler
        if (cmd.equals(ACTION_SEARCH)) { 
            //set response to not be cached
            res.setHeader("Expires","Tues, 01 Jan 1980 00:00:00 GMT");
            searchLDAP(req,out,isInternal);
        } else if (cmd.equals(ACTION_DETAIL)) { 
            if (!isInternal) {
                errorMessage(out,"Not authorized!");
            } 
            else {
                detailEdit(req,out,true);  //display only
            }
        } else if (cmd.equals(ACTION_GETCERT)) {
            binaryLDAP (req, res, certAttr, "Certificate",
                            "application/x-x509-email-cert" );
        } else if (cmd.equals(ACTION_GETPHOTO)) {
            if (!isInternal) {
                res.setContentType("text/html");
                out = res.getWriter();
                errorMessage(out,"Not authorized!");
            } 
            else {
                binaryLDAP (req, res, photoAttr, "Photograph",
                            "image/jpeg" );
            }
        } else if (cmd.equals(ACTION_EDIT)) {
            if (!isInternal) {
                res.setContentType("text/html");
                out = res.getWriter();
                errorMessage(out,"Not authorized!");
            } 
            else {
                detailEdit(req,out,false);  //edit mode 
            }
        } else if (cmd.equals(ACTION_DOEDIT)) {
            // Process user values
            modifyEntry(req,out);
        } else if (cmd.equals(ACTION_PWD)) {
            sendPwdPage(req,out,isInternal);    
        } else if (cmd.equals(ACTION_DOPWD)) {
            modifyPwd(req,out); 
        } else {
            res.setContentType("text/html");
            sendSrchPage(out,isInternal,req.getRequestURI());
        }
        return;

    }

    /**
     * Handle search requests to the servlet.<BR>
     * Retrieve arguements from the req and transmit HTML
     * back to the client.  Build an LDAP search request
     * using server side sorting when possible. If not
     * possible, do client side sorting (in the servlet).
      *
      * @param req The HTTP request
      * @param out Stream back to the client
      * @param isInternal <CODE>true</CODE> if client
      *  is internal to organization
      */
    private void searchLDAP( HttpServletRequest req, 
                             PrintWriter out,
                             boolean isInternal ) {

        // Get search arguments
        String srchType =
            readFormData(req,FLDNAME_SRCHKIND).toLowerCase();
        String srchString = readFormData(req,FLDNAME_SRCHSTRING);
        LDAPSearchResults res=null;
        LDAPConnection ld;
        String appendmsg = "";  
        String srch = "";
        String[] myAttrs;
        String uri = req.getRequestURI();
        boolean ssort = true; // User server-side sorting

        // Build full url
        String url = req.getScheme()+ "://" +
            req.getServerName();
        // Assume standard port numbers if not uncomment below
            //          + ":" + req.getServerPort();

        myAttrs = isInternal ? intAttrs : extAttrs;

        // Use the attribute name for telephone in case the
        // external number is in a different attribute than
        // the internal one.
        // myAttrs[1] is the telephone attribute name

        if ( srchType.equals("all") ) {
            srch = "(|(cn=" + srchString + "*)" +
                "(sn=" + srchString + "*)" +
                "(givenname=" + srchString+ "*)" +
                "("+myAttrs[1]+"=" + srchString + "*)" +
                ")";
        } else if ( srchType.equals("cn") ) {
            srch = "(|(cn=" + srchString + ")(cn=" +
                srchString + "*)" +
                "(cn~="+srchString+"))";
        } else if ( srchType.equals("mail") ) {
            srch = "(|(mail="+srchString+")" +
                "(mail="+srchString+"*)" +
                ")";
        } else if ( srchType.equals("telephonenumber") ) {
            srch = "(|("+myAttrs[1]+"="+srchString+")" + 
                     "("+myAttrs[1]+"="+srchString+"*)" +
                     ")";
        } else if ( srchType.equals(
            "facsimiletelephonenumber") ) {
            srch = "(|(facsimiletelephonenumber="+
                srchString+")" + 
                "(facsimiletelephonenumber="+srchString+"*)" +
                ")";
        }
        if ( (srch.length() == 0) || srchString.length() == 0) {
            // Not a valid search type
            sendSrchPage(out,isInternal,req.getRequestURI());
            return;
        }

        ld = getLDAPConn();
        try {
            LDAPSearchConstraints cons = 
                (LDAPSearchConstraints)ld.
                getSearchConstraints().clone();
            // Block till all results are in
            cons.setBatchSize( 0 );

            if ( ssort ) {
                // Use server side sorting
                LDAPSortKey sortL = new LDAPSortKey("sn");
                LDAPSortKey sortF = new LDAPSortKey("givenname");
                LDAPSortKey sortC = new LDAPSortKey("cn");
                LDAPSortKey[] sortOrder = {sortL,sortF,sortC};
                //create the server control
                LDAPSortControl sortCtl = 
                        new LDAPSortControl(sortOrder,false);
                cons.setServerControls(sortCtl);
            }

            res = ld.search( srchRoot,
                             scope,
                             srch,
                             myAttrs,
                             false,
                             cons);         

            // if sorting on server. then check if sort worked
            if ( ssort ) {
                LDAPControl[] rControls =
                    ld.getResponseControls();
                if ( rControls != null ) {
                    // Was there a server side sort error?
                    LDAPSortControl response = null;
                    for( int i = 0; i < rControls.length;
                         i++ ) {
                        if ( rControls[i] instanceof
                             LDAPSortControl ) {
                            response =
                                (LDAPSortControl)rControls[i];
                            break;
                        }
                    }
                    if ( response != null ) {
                        int resCode =
                            response.getResultCode();
                        if ( resCode !=
                             LDAPException.SUCCESS ) {
                            System.err.println(
                                "Server side sort failed!");
                            ssort = false; // Do client sort
                        }
                    }
                }
            }

            if ( !ssort ) {
                // Need to sort locally
                // Sort by sn, given and common name in
                // ascending order
                String[] sortattr = {"sn", "givenname", "cn"};
                boolean[] ascend = {true, true, true};
                res.sort( new LDAPCompareAttrNames(
                    sortattr,ascend) );
            }
        } catch ( LDAPException le ) {
            int rc = le.getLDAPResultCode();
            if (rc == LDAPException.SIZE_LIMIT_EXCEEDED) {
                appendmsg = "\nExceeded size limit";
            } else if (rc == LDAPException.TIME_LIMIT_EXCEEDED) {
                appendmsg = "\nExceeded time limit";
            } else {
                appendmsg = le.toString();
            }
        }
    
        if (res.getCount() == 0) {
            // Send an error message
            errorMessage(out,"No entries matching " 
                         + srchString + " found.");
            closeConnection(ld);
            return;
        }

        out.println("<HTML>");
        out.println("<HEAD>");  
        // JavaScript to display picture
        out.println("<SCRIPT Language=\"JavaScript\">"); 
        out.println("winopen = null;");
        out.println("");
        out.println("function displayPic(picName) {");
        out.print("   if ( (winopen != null) && ");
        out.println("(!winopen.closed) ) {");
        out.println("     winopen.location = picName;");
        out.println("     return winopen;");
        out.println("   }");
        out.println("   else {");
        out.print("     winopen = window.open(picName,'photo',");
        out.print(" 'toolbar=no,scrollbars=no,resizable=yes,");
        out.print("width="+photoWinWidth);
        out.println(",height="+photoWinHeight+"');");
        out.println("     return winopen;");
        out.println("   }");
        out.println("}");
        out.println("");
        out.println("function winClose() {");
        out.println("  if(winopen != null) winopen.close();");
        out.println("}");

        out.println("function doNothing() {}"); 
        out.println("</SCRIPT>");
        out.println("");

        out.println("<TITLE>"+orgName+" "+
                        (isInternal ? "Internal" : "")+
                        " Phonebook</TITLE>");
        out.println("</HEAD>");
        out.println("<BODY BGCOLOR="+srchOutColors+
                        "\"onUnload=\"winClose()\">");
        out.println("<CENTER><H1>"+orgName+" Search "+
                        "Results</H1></CENTER>");
        out.println("<BR><BR>");
        out.println("<TABLE WIDTH=\"100%\" BORDER=\"0\" "+
                        "CELLSPACING=\"0\">");
        out.println("<TR>");
        out.println("<TH><DIV ALIGN=\"LEFT\">Name</DIV></TH>");
        out.println("<TH><DIV ALIGN=\"LEFT\">Phone</DIV></TH>");
        out.println("<TH><DIV ALIGN=\"LEFT\">Email</DIV></TH>");
        out.println("<TH><DIV ALIGN=\"LEFT\"></DIV></TH>");
        out.println("</TR>");
        
        int rowcnt = 0;
        while ( res.hasMoreElements() ) {
            try {
                // Next directory entry
                LDAPEntry entry = res.next();
                generateRow( out,LIST_COLOR[++rowcnt%2],myAttrs,
                            entry,uri,url,isInternal);     
            } catch ( LDAPReferralException e ) {
                ; // Ignore referrals
            } catch ( LDAPException e ) {
                System.err.println( "Entry read error: " + 
                                    e.toString() );
                // There may be valid results as well
                continue;
            }
        }
        out.println("</TABLE>");
        if ( appendmsg.length() > 0 ) {
            out.println("Search error:"+appendmsg);
            out.println("<BR>");
        }
        out.println("<BR><BR>");
        out.println("<P>Questions or comments email <A HREF=\""+
                        "mailto:"+ownMail+"\">webmaster</A>"+
                        "</P>");
        out.println("</BODY>");
        out.println("</HTML>");
    
        closeConnection(ld);
        return;
    }

    /**
     * Return binary data in an HTTP stream;
     * retrieve arguments from the req 
     *
     * @param req The HTTP request
     * @param res The HTTP response
     * @param attr Name of attribute in the LDAP directory to
     *  return
     * @param name "Friendly" name of the attribute to use for
     *  transmitting an error message
     * @param mime Mime type for this binary data.
     * @exception ServletException<BR>
     * @exception IOException<BR>
     */
    private void binaryLDAP( HttpServletRequest req, 
                             HttpServletResponse res,
                             String attr, 
                             String name, String mime ) 
        throws ServletException, IOException {
        LDAPConnection ld;
        LDAPEntry theEntry = null;
        String[] attrs = {attr};
        int err = 0;
        byte[] theData = null;

        // Get form arguments
        String theDN =
            readFormData(req,FLDNAME_DN).toLowerCase();
        if ( theDN.length() == 0 ) {
            PrintWriter out = res.getWriter();
            errorMessage(out,"Error with Distingushed Name");
            return;
        }
        theDN = LDAPUrl.decode(theDN);
        ld = getLDAPConn();
        try {
            theEntry = ld.read(theDN,attrs);
            theData = getBinaryValue(theEntry,attr);
            if ( theData == null ) {
                PrintWriter out = res.getWriter();
                errorMessage(out, name + " for " + theDN + 
                             " not found!");
                err = 1;
            }
        } catch ( LDAPException e ) {
            PrintWriter out = res.getWriter();
            errorMessage(out,"Entry:" + theDN+" not found!");
            err = 1;
        } finally {
            closeConnection(ld);
        }
            
        if ( err == 1 ) {
            return;
        }

        // Prepare to transmit binary data
        res.setContentType(mime);
        // Set to pre-expire
        res.setHeader( "Expires",
                       "Tues, 01 Jan 1980 00:00:00 GMT");
        res.setContentLength(theData.length);
        ServletOutputStream sout = res.getOutputStream();
        sout.write(theData,0,theData.length);
        sout.flush();
        sout.close();

        return;
    }

    /**
      * Builds an error HTML response page<BR>
      * @param out A stream back to client
      * @param errmsg The error message
      */
    private void errorMessage( PrintWriter out, String errmsg ) {
        out.println("<HTML><HEAD>");
        out.println("<TITLE>Phonebook Error</TITLE>");
        out.println("</HEAD>");
        out.println("<BODY><BR><BR>");
        out.println("<CENTER><H1>Error</H1></CENTER><BR>");
        out.println(errmsg);
        out.println("<BR>Use the back button to correct<br>");
        out.println("</BODY></HTML>");
        return;
    }

    /**
     * Report if client is inside or outside the firewall<BR>.
     * This routine must be customized for each particular
     * use.  Currently it just matches the first three octets of an IP
     * address to determine if someone is inside the company.
     * @param req the HTTP request 
     * @return <CODE>true</CODE> if inside firewall
     */
    protected boolean checkInside( HttpServletRequest req ) {
        String clientIP = req.getRemoteAddr();
        boolean isInside = clientIP.startsWith( hostIP );
        return isInside;
    }

    /**
     * Get an LDAP Connection from the poolm
     * and assign the LDAP cache to this connection.
     * 
     * @return A valid LDAPConnection
     */
    private LDAPConnection getLDAPConn() {
        // Get connection from pool
        LDAPConnection ld = ldapPool.getConnection();
        if ( readCache != null ) {
            ld.setCache( readCache );
        }
        return ld;
    }
    
    /**
     * Return an LDAP Connection to the pool.
     * 
     * @param ld the connection to return to the pool
     */
    private void closeConnection( LDAPConnection ld ) {
        if ( ld != null ) {
            ldapPool.close( ld );
        }
    }

    /**
     * Returns the string representation of an LDAP attribute
     * from the specified entry.
     * If the attribute is not present or has a zero length
     * value then return "-". If the attribute has more than
     * one value, then just return the first one.
     * 
     * @param entry An LDAP entry
     * @param attrN Name of the attribute (e.g. "cn")
     * @return Value of attribute or -
     */
    private String getValue( LDAPEntry entry, String attrN ) {
        return getValue( entry, attrN, "-" );
    }

    /**
     * Return the string representation of an LDAP attribute
     * from the specified entry. If the attribute is not present,
     * or has a zero length value, then return the default. If
     * the attribute has more than one value, then just return
     * the first one.
     * 
     * @param entry An LDAP entry
     * @param attrN Name of the attribute (e.g. "cn")
     * @param defVal Value to return if attribute does not
     *  exist
     * @return Value of attribute or defVal
     */
    private String getValue( LDAPEntry entry,
                             String attrN,
                             String defVal) {
        LDAPAttribute attr = entry.getAttribute( attrN ); 
        if ( attr == null ) {
            return defVal;
        }
        Enumeration enumVals = attr.getStringValues();
        // Enumerate on values for this attribute
        if ( (enumVals == null) || 
             !enumVals.hasMoreElements() ) {
            return defVal;
        }
        String theValue = (String)enumVals.nextElement();
        if ( (theValue == null) ||
             (theValue.length() == 0) ) {
            return defVal;
        }
        return theValue;
    }       

    /**
     * Return the binary representation of an LDAP attribute
     * value from the specified entry.
     * If attribute is not present then return null. If the
     * attribute has more than one value, then just return
     * the first one.
     * 
     * @param entry An LDAP entry
     * @param attrN Name of the attribute (e.g. "cn")
     * @return Value of attribute as byte[] or null
     */
    private byte[] getBinaryValue( LDAPEntry entry,
                                   String attrN ) {
        LDAPAttribute attr = entry.getAttribute( attrN );
        if ( attr == null ) {
            return null;
        }
        Enumeration enumVals = attr.getByteValues();
        if ( (enumVals == null) || 
             (!enumVals.hasMoreElements()) ) {
            return null;
        }
        return (byte[])enumVals.nextElement();  
    }

    /**
     * Return the LDAP scope constant for a string
     * value of "SCOPE_BASE", "SCOPE_ONE" or
     * "SCOPE_SUB".
     * 
     * @param scope String representation of scope
     * @return LDAPConnection contant for the scope
     */
    private static int getScope( String scope ) {
        if ( scope.equalsIgnoreCase("SCOPE_BASE") ) {
            return LDAPConnection.SCOPE_BASE;
        } else if ( scope.equalsIgnoreCase("SCOPE_ONE") ) {
            return LDAPConnection.SCOPE_ONE;
        } else {
            return LDAPConnection.SCOPE_SUB;
        }
    } 

    /**
     * Return a string without attribute types in the format
     * "bjensen, people, airius.com" from a DN like
     * "uid=bjensen,ou=people,o=airius.com"
     * 
     * @param theDN A fully formed DN
     * @return A simplified DN without attribute types
     */
    private String prettyDN( String theDN ) {
        String[] dn = LDAPDN.explodeDN(theDN,true);
        String dncomps = "";
        for ( int i=0; i<dn.length; i++ ) {
            dncomps += (i==0 ? "" : ", ") + dn[i];
        }
        return dncomps;
    }

    /**
     * Return the string representing the html form value
     * from the specified post or get.
     * If form value does not exist or has no value
     * then return an empty String.
     * 
     * @param req HTTPServletRequest
     * @param fdname Name of the field in the HTML form,
     * e.g. "password"
     * @return Value of field or "" if not present
     */
    private String readFormData( HttpServletRequest req,
                                 String fdname )  {
        String d = req.getParameter(fdname);
        if ( d == null ) {
            return "";
        }
        return d.trim();
    }

    /**
     * Build an LDAPAttribute with the specified attribute
     * name and value.  If no value is specified then the
     * attribute will be deleted with an ldap replace operation.
     *
     * @param attrN Name of attribute e.g. "cn"
     * @param attrV Value for attribute, e.g. "Tom Jones"; pass
     * <CODE>null</CODE> for none
     * @return An LDAP Attribute
     *     
     */
    private LDAPAttribute buildAttr( String attrN,
                                     String attrV ) {
        if ( attrV == null ) {
            return new LDAPAttribute( attrN );
        } else {
            return new LDAPAttribute( attrN, attrV );
        }
    }

    /**
     * Handle requests for password modification by the user.
     * Validate that both new passwords are ok and that the
     * old password is correct (by binding to the directory).
     * 
     * @param req HTTPServletRequest
     * @param out Output stream
     * @exception ServletException or IOException on reading
     * or writing form data
     */
    private void modifyPwd( HttpServletRequest req, 
                            PrintWriter out )
        throws ServletException, IOException {

        LDAPConnection ld = null;
        String theDN;
        String oldpw;
        String newpw1;
        String newpw2;
        int err = 0;

        // Get detail arguments (the DN)
        theDN = readFormData(req,FLDNAME_DN).toLowerCase();
        if ( theDN.length() == 0 ) {
            errorMessage(out,"Invalid Distingushed Name");
            return;
        }
        theDN = LDAPUrl.decode(theDN);
        oldpw = readFormData(req,FLDNAME_OLDPW);
        newpw1 = readFormData(req,FLDNAME_NEWPW1);
        newpw2 = readFormData(req,FLDNAME_NEWPW2);

        // The passwords must pass the following tests:
        //   old must be valid and nonzero length
        //   new1 and new2 must match and be nonzero length
        
        if ( (!newpw1.equals(newpw2)) ||
             (newpw1.length()==0) ) {
            errorMessage(out,"Invalid new password");
            return;
        }

        if ( oldpw.length() == 0 ) {
            errorMessage(out,"Old password is invalid");
            return;
        }

        // Attempt to write change.
        // Do not use the pool for this operation because it
        // requires special credentials.

        try {
            ld = new LDAPConnection();
            ld.connect( host, port, theDN, oldpw );
            LDAPAttribute pw = 
                new LDAPAttribute("userpassword",newpw1);
            LDAPModification mod = 
                new LDAPModification(LDAPModification.REPLACE,
                                     pw);
            ld.modify( theDN, mod );
        } catch (LDAPException e) {
            err = 1;
            int num =  e.getLDAPResultCode();
            if (num == LDAPException.INVALID_CREDENTIALS) {
                errorMessage(out,"Invalid old password.");
            } else {
                errorMessage(out,e.toString());
            }
        } 

        if ( (ld!= null) && (ld.isConnected()) ) {
            try {
                ld.disconnect();
            } catch (LDAPException e) {}
        }
        if (err != 0) {
            return;
        }

        out.println("<HTML>");
        out.println("<HEAD>");
        out.println("<TITLE>"+orgName+"</TITLE>");
        out.println("<SCRIPT Language=\"JavaScript\">");
        out.println("function doNothing() {}"); 
        out.println("</SCRIPT>");
        out.println("");
        out.println("</HEAD>");
        out.println("<BODY BGCOLOR=\"white\">");
        out.println("Thank you for changing your password!");
        out.println("<BR>");
        out.print("<a href=\"javascript:doNothing()\" ");
        out.print("onclick=\"window.close();\">");
        out.println("Click Here</a>");
        out.println("</BODY>");
        out.println("</HTML>");
        
        return;
    }

    /**
      * Handle editing of an individual entry.
      * Remove attributes with no value and add or replace
      * attributes with new or different values.
      * Use the java class 
      * CParseRFC187 to parse multipart forms.
      * Multipart forms allow users to send a new photograph
      * for their entry.
      * 
      * @param req HTTPServletRequest
      * @param out Output stream
      * @exception ServletException or IOException on writing
      * changes to the directory or writing output
      */
    private void modifyEntry( HttpServletRequest req, 
                              PrintWriter out )
        throws ServletException, IOException {

        LDAPConnection ld = null;
        String theDN = "";
        String thepw = "";
        String pager = "" ;
        String mobile = "";
        String homePhone = "";
        byte[] newPhoto = null;
        int err = 0;
        
        String conType = req.getContentType().toLowerCase();
        if (conType.indexOf("multipart/form-data") == -1) {
            // Standard form submit, no picture
            // Get form elements
            theDN = readFormData(req,FLDNAME_DN).toLowerCase();
            thepw = readFormData(req,FLDNAME_OLDPW);
            pager = readFormData(req,"pager");
            mobile = readFormData(req,"mobile");
            homePhone = readFormData(req,"homePhone");
        } else {
            CParseRFC1867 parse = new CParseRFC1867(req);
            while (parse.hasMoreContent()) {
                parse.getContent();
                int nType = parse.getContentType();
                switch (nType) {
                    case CParseRFC1867.PARAMETER :
                        String fldname =
                            parse.getParameterName().
                            toLowerCase();
                        String fldval =
                            parse.getParameterValue();
                        if ( fldval == null ) {
                            fldval = "";
                        }
                        if ( fldname.equals(FLDNAME_DN) ) {
                            theDN = fldval;
                        } else if ( fldname.equals(
                            FLDNAME_OLDPW) ) {
                            thepw = fldval;
                        } else if ( fldname.equals(
                            "pager") ) {
                            pager = fldval;
                        } else if ( fldname.equals(
                            "mobile") ) {
                            mobile = fldval;
                        } else if ( fldname.equals(
                            "homephone") ) {
                            homePhone = fldval;
                        } else {
                            System.err.println(
                                "invalid field->"+fldname);
                        }
                        break;
                    case CParseRFC1867.FILE :
                        String fname =
                            parse.getFileName().trim();
                         CBlobInputStream instream = 
                             parse.getBlobInputStream();
                        if ( (fname != null) &&
                             (!fname.equals("")) ) {
                            instream.preFetch();
                            int sizePic =
                                instream.getTotalBytes();
                            if ( (sizePic <= maxPhotoSize) || 
                                 (maxPhotoSize == 0) ) {
                                // If pic size within limits
                                newPhoto = new byte[sizePic];
                                for ( int i=0;i<sizePic;i++ ) {
                                    newPhoto[i] =
                                        (byte)instream.read();
                                }
                            }
                            instream.close();
                        } else {
                            parse.readToNextBoundary();
                        } 
                        break;
                } // switch
            } // while
        }
        
        if ( theDN.length() == 0 ) {
            errorMessage(out,"Invalid Distingushed Name");
            return;
        }
        if ( thepw.length() == 0 ) {
            errorMessage(out,"Password cannot be blank");
            return;
        }
        theDN = LDAPUrl.decode(theDN);

        // Build a modification set
        LDAPModificationSet mods = new LDAPModificationSet();
        LDAPAttribute attr1 = buildAttr("pager",pager);
        LDAPAttribute attr2 = buildAttr("mobile",mobile);
        LDAPAttribute attr3 = buildAttr("homePhone",homePhone);
        
        mods.add(LDAPModification.REPLACE,attr1);
        mods.add(LDAPModification.REPLACE,attr2);
        mods.add(LDAPModification.REPLACE,attr3);
        if ( newPhoto != null ) { 
            LDAPAttribute attr4 = 
                new LDAPAttribute(photoAttr,newPhoto);  
            mods.add(LDAPModification.REPLACE,attr4);
        }

        // Attempt to write change.
        // Do not use the pool for this operation because
        // special credentials are required.
        try {
            ld = new LDAPConnection();
            ld.connect( host, port, theDN, thepw );
            ld.modify( theDN, mods );
            if ( readCache != null ) {
                readCache.flushEntries( theDN,
                                        ld.SCOPE_BASE );
            }
        } catch (LDAPException e) {
            err = 1;
            int num = e.getLDAPResultCode();
            if ( num == LDAPException.INVALID_CREDENTIALS ) {
                errorMessage(out,"Invalid Password.");
            } else {
                errorMessage(out,e.toString());
            }
        } 

        if ( (ld!= null) && (ld.isConnected()) ) {
            try {
                ld.disconnect();
            } catch (LDAPException e) {}
        }
        if (err != 0) {
            return;
        }

        out.println("<HTML>");
        out.println("<HEAD>");
        out.println("<TITLE>"+orgName+"</TITLE>");
        out.println("<SCRIPT Language=\"JavaScript\">");
        out.println("function doNothing() {}"); 
        out.println("</SCRIPT>");
        out.println("");
        out.println("</HEAD>");
        out.println("<BODY BGCOLOR=\"white\">");
        out.println("Thank you for keeping your information " +
                    "current!");
        out.println("<BR>");
        out.print("<A HREF=\"javascript:doNothing()\" ");
        out.print("onclick=\"window.close();\">");
        out.println("Click Here</a>");
        out.println("</BODY>");
        out.println("</HTML>");

        return;
    }

    /**
     * Handles requests for search page or requests that
     * are not valid (we send the search page).
     * 
     * @param out output stream
     * @param isInternal <CODE>true</CODE> if internal user
     * @param uri The servlet's uniform resource identifier
     *               
     */
    private void sendSrchPage( PrintWriter out, 
                               boolean isInternal,
                               String uri ) {
        // Dump the search page

        out.println("<HTML>");
        out.println("<HEAD>");
        out.println("<TITLE>"+orgName+" "+
                    (isInternal ? "Internal" : "")+
                    " Phonebook</TITLE>");
        out.println("</HEAD>");
        out.println("<BODY BGCOLOR="+srchPageColors+">");
        out.println("<H2 ALIGN=\"center\">"+orgName+
                    " Phonebook</H2>");
        out.println("<P>The "+orgName+" Phone Book allows you"+
                    " to locate an individual within the "+
                    "company.");
        out.println("Please enter your search ");
        out.println("string below. You may search for phone "+
                    "number, email, etc....</P>");
        out.print("<FORM METHOD=\"post\" action=\""+ uri + "?");
        out.println(ACTION_FIELD+"="+ACTION_SEARCH+"\">");
        
        out.println("<CENTER>");
        out.println("<TABLE WIDTH=\"75%\" BORDER=\"0\">");
        out.println("<TR>");
        out.println("<TD>");
        out.println("<DIV ALIGN=\"right\">Search:</DIV>");
        out.println("</TD>");
        out.println("<TD>");
        out.println("<SELECT NAME="+FLDNAME_SRCHKIND+">");
        out.println("<OPTION VALUE=\"all\">Anything</OPTION>");
        out.println("<OPTION VALUE=\"cn\" selected>"+
                    "Name</OPTION>");
        out.println("<OPTION VALUE=\"mail\">"+
                    "Electronic Mail</OPTION>");
        out.println("<OPTION VALUE=\"telephonenumber\">"+
                    "Phone Number</OPTION>");
        out.println("<OPTION VALUE=\""+
                    "facsimiletelephonenumber\">"+
                    "Fax Number</OPTION>");
        out.println("</SELECT>");
        out.println("</TD>");
        out.println("</TR>");
        out.println("<TR>");
        out.println("<TD>&nbsp;</TD>");
        out.println("<TD>");
        out.println("<INPUT TYPE=\"text\" name="+
                    FLDNAME_SRCHSTRING+" size=\"40\""+
                    " maxlength=\"60\">");
        out.println("</TD>");
        out.println("</TR>");
        out.println("</TABLE>");
        out.println("</CENTER>");
        out.println("<BR>");
        out.print("<CENTER><INPUT TYPE=\"submit\" ");
        out.println("name=\"Submit\" value=\"Search\">"+
                    "</CENTER>");
        out.println("</FORM>");

        out.println("<BR>");
        out.println("<P>Questions or comments email "+
                    "<A HREF=\"mailto:"+ownMail+"\">"+
                    "webmaster</A></P>");
        out.println("</BODY>");
        out.println("</HTML>");

        return;
    }

    /**
     * Handles requests for password change page
     * 
     * @param req HTTPServlet Request
     * @param out output stream
     * @param isInternal <CODE>true</CODE> if internal user
     * @exception throws ServletException or IOException for
     * invalid URL or error on writing output
     *               
     */
    private void sendPwdPage( HttpServletRequest req,
                              PrintWriter out, 
                              boolean isInternal )
            throws ServletException, IOException {

        String theDN;
        String theDNd;
        String uri = req.getRequestURI();

        // Get the DN of the record we are altering
        theDN = readFormData(req,FLDNAME_DN).toLowerCase();
        theDN = LDAPUrl.decode(theDN);
        if ( theDN.length() == 0 ) {
            errorMessage(out,"Error with Distingushed Name");
            return;
        }
        
        out.println("<HTML>");
        out.println("<HEAD>");
        out.println("<TITLE>"+orgName+"</TITLE>");

        out.println("<SCRIPT Language=\"JavaScript\">");
        out.println("function doNothing() {}"); 
        out.println("</SCRIPT>");
        out.println("");

        out.println("</HEAD>");
        out.println("<BODY BGCOLOR=\"white\">");
        out.print("<FORM METHOD=POST name=\"PwdModForm\""); 
        out.print("action=\""+uri+"?"+ACTION_FIELD+"=");
        out.println(ACTION_DOPWD+"\">");
        out.print("<INPUT TYPE=\"hidden\" name=\""+FLDNAME_DN);
        out.println("\" value=\""+LDAPUrl.encode(theDN)+"\">");

        String dncomps = prettyDN(theDN);
        out.println("<FONT SIZE=\"+2\">"+dncomps+"</FONT><BR>");
        out.println("<BR>");

        out.print("<TABLE CELLSPACING=\"2\" ");
        out.println("BORDER BGCOLOR=#F2F2F2 WIDTH=60%>");
        out.println("<TR>");
        out.print("<TD BGCOLOR=#006666>");
        out.print("<FONT FACE=ARIAL,HELVETICA COLOR=WHITE>");
        out.println(" New Password</FONT></TD>");
        out.println("</TR>");
        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\" NOWRAP>");
        out.print("<INPUT TYPE=\"password\" name=");
        out.print(FLDNAME_NEWPW1+" size=\"20\" ");
        out.println("MAXLENGTH=\"20\">");
        out.println("</TD>");
        out.println("</TR>");
        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\" NOWRAP>");
        out.print("<INPUT TYPE=\"password\" NAME=");
        out.print(FLDNAME_NEWPW2+" size=\"20\" ");
        out.println("MAXLENGTH=\"20\">");
        out.println("Confirm New Password</TD>");
        out.println("</TR>");
        out.println("</TABLE>");
        out.println("<BR>");
        out.print("<TABLE CELLSPACING=\"2\" BORDER ");
        out.println("BGCOLOR=#F2F2F2 WIDTH=60%>");
        out.println("<TR>");
        out.print("<TD BGCOLOR=#006666>");
        out.print("<FONT FACE=ARIAL,HELVETICA COLOR=WHITE>");
        out.println(" Old Password");
        out.println("</FONT></TD>");
        out.println("</TR>");
        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\" NOWRAP>");
        out.print("<INPUT TYPE=\"password\" NAME=");
        out.print(FLDNAME_OLDPW+" SIZE=\"20\" ");
        out.println("MAXLENGTH=\"20\">");
        out.println("</TD>");
        out.println("</TR>");
        out.println("</TABLE>");
        out.println("<BR>");
        out.print("<CENTER><INPUT TYPE=\"submit\" NAME=\"");
        out.print("submit\" VALUE=\"Change Password");
        out.println("\"></CENTER>");
        out.println("</FORM>");
        out.println("<BR>");
        out.print("<A HREF=\"javascript:doNothing()\" ");
        out.print("onclick=\"window.close();\">");
        out.println("Cancel Password Modify</A>");

        out.println("</BODY>");
        out.println("</HTML>");

        return;
    }

    /**
     * Generates a single row for the search results page.
     * 
     * @param out PrintWriter for output stream
     * @param color is background color for this row
     * @param attrs the names of the attributes for this row
     *  for each of the 5 columns.
     * @param entry containing the data for this row
     * @param uri is the servlet's uniform resource identifier
     * @param url complete url to get back to servlet
     * @param isInternal true if internal user
     *               
     */
    private void generateRow( PrintWriter out, String color,
                              String[] attrs, LDAPEntry entry, 
                              String uri, String url, 
                              boolean isInternal ) {
        // Generate one row of output data
        String theDN = LDAPUrl.encode(entry.getDN()); // Encode
        String theName = getValue(entry,attrs[0]);
        String thePhone = getValue(entry,attrs[1]);
        String theMail = getValue(entry,attrs[2]);
        String certimg = "&nbsp;"; 
        String photoimg = "&nbsp;";
        String nameurl = theName;

        // Most of this is building the custom line based on 
        // whether the caller is an external or internal user.
        // External users do not get to click on the name for
        // details, among other things.
        byte[] theCert = getBinaryValue(entry,certAttr);
        if ( theCert != null ) { // Has cert so give url
            certimg = "<A HREF=\""+uri+"?"+ACTION_FIELD+"="+
                ACTION_GETCERT+
                "&"+FLDNAME_DN+"="+theDN+"\"><IMG SRC=\""+
                imagesURL+
                "cert.gif\" WIDTH=24 HEIGHT=21 BORDER=0 " + 
                "alt=\"Get digital certificate\"></A>";
        }

        byte[] thePhoto = getBinaryValue(entry,attrs[4]);
        if (thePhoto != null && isInternal) {
            photoimg = "<A HREF=\"javascript:doNothing()\" " +
                "onclick=\"winopen=displayPic('"+url+uri+"?"+
                ACTION_FIELD+"="+ACTION_GETPHOTO+"&"+
                FLDNAME_DN+"="+theDN+"');\"><IMG SRC=\""+imagesURL+
                "photo.gif\" WIDTH=37 HEIGHT=21 BORDER=0 " + 
                "alt=\"Look at Photograph\"></A>";
        }

        if (isInternal) {
            nameurl = "<A HREF=\""+uri+"?"+ACTION_FIELD+"="+
                 ACTION_DETAIL+
                "&"+FLDNAME_DN+"="+theDN+"\">"+theName+"</A>";
        }

        //colors are either "#666600" or "#009999"
        out.println("<TR BGCOLOR=\""+color+"\">");
        out.println("<TD>"+nameurl+"</TD>");
        out.println("<TD>"+thePhone+"</TD>");
        if (theMail.equals("-")) {
            out.println("<TD> - </TD>");
        } else {
            out.println("<TD><A HREF=\"mailto:" + theMail + "\">" +
                theMail + "</A></TD>");
        }
        out.println("<TD>"+certimg+"&nbsp;"+photoimg+"</TD>");
        out.println("</TR>");
    }

    /**
      * Handles requests for detail view and edit pages
        * 
        * @param req HTTPServletRequest
        * @param out PrintWriter
        * @param detail <CODE>true</CODE> if detail view
        * (view only)
        * if edit page.
        * @exceptions ServletException or IOException on reading
        * from directory or writing output
        *               
      */
    private void detailEdit( HttpServletRequest req, 
                             PrintWriter out,
                             boolean detail )
        throws ServletException, IOException {

        LDAPConnection ld;
        LDAPEntry theEntry = null;
        String theDN;
        int err = 0;
        String uri = req.getRequestURI();
        String imgURL = "";

        // If detail is true then show the record and
        // allow editing of certain fields.

        // Get detail arguments (the DN)
        theDN = readFormData(req,FLDNAME_DN).toLowerCase();
        if ( theDN.length() == 0 ) {
            errorMessage(out,"Error with Distingushed Name");
            return;
        }
        theDN = LDAPUrl.decode(theDN);
        ld = getLDAPConn();
        try {
            theEntry = ld.read(theDN);  // Read entry
        } catch ( LDAPException e ) {
            errorMessage(out,"Entry:" + theDN+" not found!");
            closeConnection(ld);
            return;
        }

        String cn = getValue(theEntry,"cn");
        String dncomps = prettyDN(theEntry.getDN());        

        // Format a detail record for this entry
        out.println("<HTML><HEAD>");
        if ( detail ) {
            out.println("<TITLE>Detail for "+cn+"</TITLE>");
        } else {
            out.println("<TITLE>Edit for "+cn+"</TITLE>");
        }
        out.println("</HEAD>");
        out.println("<BODY BGCOLOR=\"#FFFFFF\">");
        out.println("<FONT SIZE=\"+2\">"+dncomps+"</FONT>");
        out.println("<BR>");
        //if edit put a form tag in there
        if ( !detail ) {
            out.print("<FORM METHOD=\"post\" ACTION=\"");
            out.print(uri+"?"+ACTION_FIELD+"="+ACTION_DOEDIT);
            out.println("\" ENCTYPE=\"multipart/form-data\">");
        }

        if ( detail ) {
            out.println("<DIV ALIGN=\"right\">");
            out.println("<A HREF=\""+uri+"?"+ACTION_FIELD+"="+
                        ACTION_EDIT+"&"+FLDNAME_DN+"="+
                        LDAPUrl.encode(theDN)+"\" "+
                        "TARGET=\"_blank\">"+
                        "<IMG SRC=\""+imagesURL+
                        "edit.gif\" alt=\"Edit this entry.\">"+
                        "</A>");
            out.println("<A HREF=\""+uri+"?"+ACTION_FIELD+"="+
                        ACTION_PWD+"&"+FLDNAME_DN+"="+
                        LDAPUrl.encode(theDN)+"\" "+
                        "TARGET=\"_blank\">"+
                        "<IMG SRC=\""+imagesURL+
                        "pwd.gif\" alt=\"Change password.\">"+
                        "</A></DIV>");
        }

        out.println("<BR>");
        out.println("<TABLE CELLSPACING=\"2\" " +
                        "BORDER BGCOLOR=#F2F2F2 WIDTH=95%>");
        out.println("<TR>");
        out.println("<TD BGCOLOR=#006666 COLSPAN=4>"+
                    "<FONT FACE=ARIAL,HELVETICA COLOR=WHITE>"+
                    "Contact Information</FONT></TD>");
        out.println("</TR>");

        out.println("<TD VALIGN=\"top\" NOWRAP>First Name:"+
                    "</TD>");
        out.println("<TD VALIGN=\"top\" NOWRAP><B>"+
                    getValue(theEntry,"givenname")+
                    "</B></TD>");
        out.println("<TD VALIGN=\"top\" NOWRAP ROWSPAN=\"2\""+
                    ">Full Name:</TD>");
        out.println("<TD VALIGN=\"top\" NOWRAP ROWSPAN=\"2\""+
                    "><B>"+cn+"</B></TD></TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"top\" NOWRAP>Last Name:"+
                    "</TD>");
        out.println("<TD VALIGN=\"top\" NOWRAP><B>"+
                    getValue(theEntry,"sn")+"</B></TD></TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\">Phone:</TD>");
        out.println("<TD VALIGN=\"TOP\" NOWRAP><B>"+
                    getValue(theEntry,"telephonenumber")+
                    "</B></TD>");
        out.println("<TD VALIGN=\"TOP\" NOWRAP>"+
                    "E-Mail Address:</TD>");

        String email = getValue(theEntry,"mail");
        if ( !email.equals("-") ) {
            email = "<A HREF=mailto:"+email+">"+email+"</A>";
        }
        out.println("<TD VALIGN=\"TOP\" NOWRAP><B>"+
                    email+"</B></TD></TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\">Fax:</TD>");
        out.println("<TD VALIGN=\"TOP\" nowrap><B>"+
                    getValue(theEntry,"facsimiletelephonenumber")
                    +"</B></TD>");
        out.println("<TD VALIGN=\"TOP\" nowrap>User ID:</TD>");
        out.println("<TD VALIGN=\"TOP\" nowrap><B>"+
                    getValue(theEntry,"uid")+"</B></TD></TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\" nowrap>Pager:</TD>");

        out.println("<TD VALIGN=\"TOP\" nowrap><B>");

        if ( detail ) {
            out.println(getValue(theEntry,"pager")+"</B></TD>");
        } else {
            out.println("<input type=\"text\" name=\"pager\" "+
                            "size=\"20\" maxlength=\"20\" value=\""+
                            getValue(theEntry,"pager","")+"\"></B></TD>");
        }

        out.println("<TD VALIGN=\"TOP\" nowrap>Mobile Phone:"+
                        "</TD>");

        out.println("<TD VALIGN=\"TOP\" nowrap><B>");
        if ( detail ) {
            out.println(getValue(theEntry,"mobile")+"</B></TD></TR>");
        } else {
            out.println("<input type=\"text\" name=\"mobile\" "+
                        "size=\"20\" maxlength=\"20\" value=\""+
                        getValue(theEntry,"mobile","")+"\">"+
                        "</B></TD></TR>");
        }

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\" nowrap>Home Phone:</TD>");

        out.println("<TD VALIGN=\"TOP\" nowrap><B>");
        if ( detail ) {
            out.println(getValue(theEntry,"homePhone")+"</B>"+
                            "</TD>");
        } else {
            out.println("<input type=\"text\" name=\"homePhone\" "+
                            "size=\"20\" maxlength=\"20\" value=\""+
                            getValue(theEntry,"homePhone","")+"\">"+
                            "</B></TD>");
        }
        // Display photograph if present
        out.println("<TD VALIGN=\"TOP\" nowrap>Photograph:</TD>");
        out.print("<TD VALIGN=\"TOP\" nowrap>");
        if ( getBinaryValue(theEntry,photoAttr) != null ) {
            imgURL = "<IMG SRC=\""+uri+"?"+ACTION_FIELD+"="+
            ACTION_GETPHOTO+"&"+FLDNAME_DN+"="+
            LDAPUrl.encode(theDN)+"\" width=\"70\" height=\"70\">";
        } else {
            imgURL = "<B>-</B>";
        }
        
        if ( !detail ) {
            //put a browse for new picture button
            imgURL += "<BR><INPUT TYPE=FILE NAME="+
                FLDNAME_NEWPHOTO+">";
        }

        imgURL += "</TD>";
        out.println(imgURL);
        out.println("</TABLE>");
        out.println("<BR><BR>");

        // Business and location information in this section

        out.println("<TABLE CELLSPACING=\"2\" " +
                    "BORDER BGCOLOR=#F2F2F2 WIDTH=95%>");
        out.println("<TR>");
        out.println("<TD BGCOLOR=#006666 COLSPAN=4>"+
                    "<FONT FACE=ARIAL,HELVETICA COLOR=WHITE>"+
                    "Business and "+
                    "Location Information</FONT>"+
                    "</TD>");
        out.println("</TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\">Building:</TD>");
        out.println("<TD VALIGN=\"TOP\" nowrap><B>"+
                    getValue(theEntry,"buildingName")+
                    "</B></TD>");
        out.println("<TD VALIGN=\"TOP\">Title:</TD>");
        out.println("<TD VALIGN=\"TOP\" nowrap><B>"+
                    getValue(theEntry,"title")+"</B></TD>");
        out.println("</TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\">Manager:</TD>");

        String mgr = getValue(theEntry,"manager");
        String mgrPhone = "-";

        if (!mgr.equals("-")) {
            String[] mgrAttrs = {"cn","telephoneNumber"};
            try {
                LDAPEntry mgrEntry = ld.read(mgr,mgrAttrs);
                String mgrdn = LDAPUrl.encode(mgr);
                String mgrName = getValue(mgrEntry,mgrAttrs[0]);
                mgrPhone = getValue(mgrEntry,mgrAttrs[1]);
                mgr = "<A HREF=\""+uri+"?"+ACTION_FIELD+"="+
                    ACTION_DETAIL+"&"+FLDNAME_DN+"="+
                    mgrdn+"\">"+mgrName+"</A>";
            } catch (LDAPException e) {
                mgr = "-";
            }
        }

        out.println("<TD VALIGN=\"TOP\" NOWRAP><B>"+
                    mgr+"</B></TD>");
        out.println("<TD VALIGN=\"TOP\">Mgr. Phone:</TD>");
        out.println("<TD VALIGN=\"TOP\" NOWRAP><B>"+
                    mgrPhone+"</B></TD>");
        out.println("</TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\">City:</TD>");
        out.println("<TD VALIGN=\"TOP\"><B>"+
                    getValue(theEntry,"l")+"</B></TD>");
        out.println("<TD VALIGN=\"TOP\">State:</TD>");
        out.println("<TD VALIGN=\"TOP\"><B>"+
                    getValue(theEntry,"st")+"</B></TD>");
        out.println("</TR>");

        out.println("<TR>");
        out.println("<TD VALIGN=\"TOP\">Mailing Address:</TD>");
        out.println("<TD VALIGN=\"TOP\" COLSPAN=\"4\" "+
                    "NOWRAP><B>");
        String postAddr = getValue(theEntry,"l","")+", "+
                          getValue(theEntry,"st","")+" "+
                          getValue(theEntry,"postalCode","");
        if (postAddr.length() < 5) { //none stored
            postAddr = "";
        }
        out.println(getValue(theEntry,"postalAddress")+"<BR>");
        out.println(postAddr+"</B></TD>");
        out.println("</TR>");

        out.println("</TABLE>");

        if (!detail) {

            // Prompt for password
            out.println("<BR> Password: ");
            out.println("<INPUT TYPE=\"password\" name=\""+
                        FLDNAME_OLDPW+"\" size=\"20\""+
                        " maxlength=\"20\"><BR>");

            out.println("<BR><CENTER>");
            out.println("<INPUT TYPE=\"submit\" NAME=\"Submit\""+
                        " VALUE=\"Submit\">");
            out.println("<INPUT TYPE=\"reset\" NAME=\"Reset\""+
                        " VALUE=\"Reset\">");
            out.println("</CENTER><BR>");
            out.print("<INPUT TYPE=\"hidden\" NAME=\""+
                      FLDNAME_DN);
            out.println("\" VALUE=\""+LDAPUrl.encode(theDN)+
                        "\">");
            out.println("</FORM>");
        }

        out.println("</BODY></HTML>");
        closeConnection(ld);
    }

}
