import java.io.*;
import netscape.ldap.client.JDAPBERTagDecoder;
import netscape.ldap.LDAPControl;
import netscape.ldap.LDAPException;
import netscape.ldap.ber.stream.*;

/**
 * Represents control data for returning the results of a modification
 * when an email report was requested.
 *
 *     ModificationEmailResponse ::= SEQUENCE {
 *            resultCode     INTEGER,
 *            emailMessage   OCTET STRING
 *     }
 *
 */

public class EmailResponseControl extends LDAPControl {
    public final static String EMAILRESPONSE = "5.5.5.5.5.2";

    /**
     * Constructs a new <CODE>EmailResponse</CODE> object.
     * @param oid Must be EmailRequestControl.EMAILREQUEST
	 * @param critical Not used in the control
     * @param value A BER encoded byte array.
     * @exception netscape.ldap.LDAPException If oid is not 
     * <CODE>EmailResponseControl.EMAILRESPONSE</CODE>.
     * @see netscape.ldap.LDAPControl
     */
    public EmailResponseControl( String oid, boolean critical,
								 byte[] value ) throws LDAPException {
        super( EMAILRESPONSE, true, null );
        if ( !oid.equals( EMAILRESPONSE ) ) {
             throw new LDAPException( "oid must be " +
									  "EmailResponseControl." +
                                      "EMAILRESPONSE", 
                                      LDAPException.PARAM_ERROR);
        }
        m_value = value;
        parseResponse();
    }

    /**
     * Gets the result code from delivering an email message
     * @return The result code from delivering an email message
     */
    public int getCode() {
        return m_code;
    }

    /**
     * Gets the status report delivered with the email message
     * @return The status report delivered with the email message
     */
    public String getMessage() {
        return m_message;
    }

    /**
     * Parses the BER-encoded value of m_value.
     */
    private void parseResponse() {
        /* Suck out the data and parse it */
        ByteArrayInputStream inStream =
            new ByteArrayInputStream( getValue() );
        BERSequence ber = new BERSequence();
        JDAPBERTagDecoder decoder = new JDAPBERTagDecoder();
        int[] nRead = new int[1];
        nRead[0] = 0;
        try  {
            /* A sequence */
            BERSequence seq = (BERSequence)BERElement.getElement(
                                                      decoder, inStream,
                                                      nRead );
            /* First is the result code, then the message */
            m_code = ((BERInteger)seq.elementAt( 0 )).getValue();
			BEROctetString str = (BEROctetString)seq.elementAt( 1 );
			m_message = new String(str.getValue(), "UTF8");
        } catch(Exception x) {
            m_code = -1;
            m_message = null;
        }
    }

    private int m_code = -1;
    private String m_message = null;
}
