/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import javax.swing.*;
import javax.swing.event.*;
import netscape.ldap.*;
import netscape.ldap.controls.*;

/**
 * A Java application that presents a scrolling list of sorted names from a
 * directory. There is a text field for typedown - the list updates itself and
 * scrolls to the first name matching the characters in the text field.
 */
public class VListDialog extends JDialog implements DocumentListener {
    /**
     * All configuration is done through parameters in the constructor
     *
     * @param ldc A sufficiently authenticated connection to a server
     * @param base The base DN for the search
     * @param scope The search scope
     * @param filter The search filter
     * @param sortAttrs One or more attributes to sort by
     */
    public VListDialog( LDAPConnection ldc, String base, int scope,
                  String filter, String[] sortAttrs ) {
        super( new JFrame(), true );

        VListPanel listPanel =
            new VListPanel( ldc, base, scope, filter, sortAttrs );

        // The panel is to be notified of changes in the typedown field
        addPropertyChangeListener( listPanel );

        initUI( listPanel, ldc.getHost(), ldc.getPort(), base, filter );
        setTitle( base );
    }

    /**
     * Add a label and a disabled text field to the panel.
     *
     * @param panel The container for the label and text field
     * @param gbc Layout constraints
     * @param label A constructed label
     * @param field A constructed text field
     */
    private void addTextField( Container panel, GridBagConstraints gbc,
                               JLabel label, JTextField field ) {
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.gridwidth = 1;
        gbc.weightx = 0;
        gbc.insets = new Insets(3,PAD,1,8);
        gbc.fill = gbc.NONE;
        panel.add( label, gbc );

        field.setEnabled( false );
        gbc.gridx = 1;
        gbc.weightx = 1;
        gbc.gridwidth  = gbc.REMAINDER;
        gbc.insets = new Insets(3,0,1,PAD);
        gbc.fill = gbc.HORIZONTAL;
        panel.add(field, gbc);
    }

    /**
     * Create all the user interface components and lay them out
     *
     * @param listPanel A panel containing a search results list
     * @param host Name of LDAP host
     * @param port Port of LDAP host
     * @param base Base DN for search
     * @param filter Search filter
     */
    protected void initUI( JComponent listPanel, String host, int port,
                           String base, String filter ) {
        // Add the result list panel in a scrolling pane
        Container panel = getContentPane();
        panel.setLayout( new GridBagLayout() );
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridwidth = gbc.REMAINDER;
        gbc.gridheight = 1;
        gbc.fill = gbc.BOTH;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weighty = 1.0;
        JScrollPane scrollPane = new JScrollPane(listPanel);
        panel.add(scrollPane, gbc);

        // Layout rules for labels and text fields
        gbc.anchor = gbc.WEST;
        gbc.weighty = 0;

        // Add the read-only fields showing the parameters used
        addTextField( panel, gbc,
                      new JLabel("Host: "), new JTextField(host) );
        String portName = Integer.toString( port );
        addTextField( panel, gbc,
                      new JLabel("Port: "), new JTextField(portName) );
        addTextField( panel, gbc,
                      new JLabel("Base: "), new JTextField(base) );
        addTextField( panel, gbc,
                      new JLabel("Filter: "), new JTextField(filter) );

        // Add the field where the user can type in
        _typedown = new JTextField( "" );
        addTextField( panel, gbc, new JLabel("Typedown: "), _typedown );
        _typedown.setEnabled( true );
        _typedown.getDocument().addDocumentListener( this );
        _typedown.setRequestFocusEnabled( true );
        _typedown.requestFocus();

        /* Cancel if the window is closed */
        addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    setVisible(false);
                    dispose();
                }
            }
        );
    }

    /**
     * Implement the DocumentListener interface.<BR>
     * Catch all changes in the typedown text field and update the
     * scrolling list.
     *
     * @param e The event from the typedown text field Document
     */
    public void changedUpdate(DocumentEvent e) {
        doTypedown();
    }
    public void removeUpdate(DocumentEvent e) {
        doTypedown();
    }
    public void insertUpdate(DocumentEvent e) {
        doTypedown();
    }

    /**
     * Tell the model to do typedown, and then scroll the list so the
     * selected index is visible
     */
    private void doTypedown() {
        firePropertyChange( "typedown", _oldText, _typedown.getText() );
        _oldText = _typedown.getText();
    }

    /**
     * Add a client to be notified when an authentication result is in
     * @param listener a client to be notified of changes
     */
    public void addPropertyChangeListener( PropertyChangeListener listener ) {
        _propSupport.addPropertyChangeListener( listener );
    }

    /**
     * Remove a client which had requested notification on authentication
     * @param listener a client to not be notified of changes
     */
    public void removePropertyChangeListener(
                              PropertyChangeListener listener ) {
        _propSupport.removePropertyChangeListener( listener );
    }

    /**
     * Support for bound property notification
     * @param propName Name of changed property
     * @param oldValue Previous value of property
     * @param newValue New value of property
     */
    public void firePropertyChange( String propName,
                                    Object oldValue,
                                    Object newValue ) {
        _propSupport.firePropertyChange( propName, oldValue, newValue );
    }

    private static final int PAD = 10;
    private JTextField _typedown;
    private boolean _skipVLV;
    private boolean _skipSort;
    private String _oldText = "";
    private PropertyChangeSupport _propSupport =
              new PropertyChangeSupport( this );

    /**
     * Parse command line arguments and launch a dialog with a VLV pane
     */
    public static void main( String[] args ) {
        if ( args.length < 5 ) {
            System.out.println( "Usage: VList <host> <port> <baseDN> " +
                                "<filter> <sortAttr(s)> [bindDN password]" );
            System.exit(1);
        }
        String bindDN = (args.length > 5) ? args[5] : "";
        String bindPassword = (args.length > 6) ? args[6] : "";
        int scope = LDAPConnection.SCOPE_SUB; // Hard-coded here
        String base = args[2];
        String filter = args[3];

        // There may be more than one attribute to sort by
        StringTokenizer st = new StringTokenizer( args[4] );
        String[] sortAttrs = new String[st.countTokens()];
        int i = 0;
        while( st.hasMoreTokens() ) {
            sortAttrs[i++] = st.nextToken();
        }

        // Connect to the server
        LDAPConnection ldc = null;
        try {
            ldc = new LDAPConnection();
            String host = args[0];
            int portNumber = Integer.parseInt(args[1]);
            System.out.println( "Connecting to " + host +
                                ":" + portNumber );
            ldc.connect( 3, host, portNumber, bindDN, bindPassword );
        } catch ( LDAPException e ) {
            System.out.println( e );
            System.exit( 1 );
        }
        VListDialog vlist = new VListDialog( ldc, base, scope, filter,
                                             sortAttrs );
        vlist.setBounds( new Rectangle( 100, 100, 300, 300 ) );
        vlist.show();
        System.exit(0);
    }
}
