/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.*;
import java.io.*;
import netscape.ldap.*;

/**
 * Class that reads a comma-delimited file of user records and
 * inserts the records into an ldap directory at a specified
 * base DN.
 */
public class MultiAdd {

    /**
     * Add entries from a comma-delimited file of user records
     *
     * @param args host, port, filename, dn, authdn,
     * password
     */
    public static void main( String[] args ) {

        if ( args.length != 6 ) {
            System.out.println( "Usage: java MultiAdd " +
                                "<host> <port> <filename> " +
                                "<dn> <authdn> <password> " );
            System.out.println( "Example:" );
            System.out.println( "    java MultiAdd " +
                                "localhost 389 " +
                                "records.csv " +
                                "o=airius.com\" " +
                                "\"cn=Directory Manager\" " +
                                "password" );
            System.out.println( "The comma-delimited file " +
                                "should contain:");
            System.out.println("  uid,fname,lname,phone" );
            System.exit(1);
        }

        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String filename = args[2];
        String basedn = args[3];
        String authid = args[4];
        String authpw = args[5];

        // Objectclasses for a person entry
        String[] objclass = { "top","person",
                              "organizationalPerson",
                              "inetOrgPerson"};
        LDAPAttribute attrobj =
            new LDAPAttribute( "objectclass", objclass );

        BufferedReader in = null;
        int numAdded = 0;
        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect to server and authenticate
            ld.connect( host, port, authid, authpw );
            System.out.println( "Connected to " + host );

            in = new BufferedReader(
                new FileReader(filename));

            String line;
            while ( (line = in.readLine()) != null ) {
                String uid = "";
                String fname = "";
                String lname = "";
                String phone = "";
                // Parse the line
                try {               
                    StringTokenizer t =
                        new StringTokenizer(line,",");
                    uid = t.nextToken();
                    fname = t.nextToken();
                    lname = t.nextToken();
                    phone = t.nextToken();
                } catch (NoSuchElementException e) {
                    System.err.println( "Missing fields in: " +
                                        line );
                    continue;
                }

                String dn = "uid=" + uid + "," + basedn;
                LDAPAttributeSet attrs =
                    new LDAPAttributeSet();
                attrs.add( attrobj );
                // Western composition: firstname, lastname
                String fullname = fname + " " + lname;
                attrs.add( new LDAPAttribute("cn", fullname) );
                attrs.add( new LDAPAttribute(
                                  "givenname", fname) );
                attrs.add( new LDAPAttribute("sn",lname) );
                attrs.add( new LDAPAttribute(
                                  "telephonenumber", phone) );
                // Now add the record to the directory
                LDAPEntry theEntry = new LDAPEntry( dn, attrs );
                try {
                    ld.add( theEntry );
                    System.out.println("Added: " + dn);
                    numAdded++;
                } catch ( LDAPException lda ) {
                    System.out.println( lda.toString() );
                    System.out.println("Failed to add: " + dn);
                }
            }
        
        } catch( Exception e ) {
            // Either IOException on reading the file or
            // LDAPException on connecting to LDAP server
            System.out.println( e.toString() );
        }

        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
            try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }
        if ( in != null ) {
            try {
                in.close();
            } catch ( IOException e ) {}
        }
        System.out.println( numAdded + " entries added" );
        System.exit(0);
    }
}


