/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.io.*;
import java.util.Date;
import java.util.*;
import netscape.ldap.*;

/**
 * Read or write the serialized form of a preferences object
 * to the directory
 */
public class SavePrefs {
    private static final int NONE = 0;
    private static final int READ = 1;
    private static final int WRITE = 2;

    // Where to store the preferences
    private static final String PREFS_DN =
        "cn=savePrefs,ou=people,o=airius.com";

    private static final String ATTR_CLASS =
        "javaClassName";
    private static final String ATTR_DATA =
        "javaSerializedData";

    // Objectclasses of the preferences object
    private static final String[] PREFS_OC =
        { "top", "javaContainer",
          "javaSerializedObject" };

    /**
     * Read or write the serialized form of a preferences
     * object to "cn=savePrefs,ou=people,o=airius.com"
     *
     * @param args host, port, dn, authdn, password,
     * [r] | [w, x, y]
     */
    public static void main (String[] args) {

        int pgmOption = NONE;
        Prefs prefvals = null;
        String host = null;
        int port = -1;
        String authid = null;
        String authpw = null;
        int err = -1;

        if ( args.length > 4 ) {
            host = args[0];
            port = Integer.parseInt( args[1] );
            authid = args[2];
            authpw = args[3];
            if ( (args.length == 5) &&
                 args[4].equalsIgnoreCase("r") ) {
                pgmOption = READ;
            } else if ( (args.length == 7) &&
                        args[4].equalsIgnoreCase("w") ) {
                prefvals =
                    new Prefs( Integer.parseInt(args[5]),
                                  Integer.parseInt(args[6]) );
                pgmOption = WRITE;
            }
        }
        if ( pgmOption == NONE ) {
            System.out.println( "Usage: java SavePrefs " +
                                "<host> <port> " +
                                "<authdn> <password> " +
                                "[r]|[w x y]" );
            System.out.println( "Examples:" );
            System.out.println( "    java SavePrefs " +
                                "localhost 389 " +
                                "\"cn=Directory Manager\" " +
                                "password r" );
            System.out.println( "    java SavePrefs " +
                                "localhost 389 " +
                                "\"cn=Directory Manager\" " +
                                "password w 35 62" );
            System.out.println("r reads values");
            System.out.println("w writes values x and y");
            System.exit(1);
        }


        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect to server and authenticate
            ld.connect( host, port, authid, authpw );
        } catch ( LDAPException e ) {
            System.out.println("Error: " + e.toString());
            System.exit(1);
        }

        if ( pgmOption == READ ) {
            // Read data from directory
            byte[] info = readBytes( ld, PREFS_DN, ATTR_DATA );
            if ( info == null ) { 
                System.out.println( "Unable to read data " +
                                    "from " + PREFS_DN );
            } else {    
                // Deserialize the data into an object
                try {           
                    prefvals =
                        (Prefs)deserializeObject( info );
                    System.out.println( "a=" +
                                        prefvals.geta() );
                    System.out.println( "b=" +
                                        prefvals.getb() );
                    System.out.println( "Date created = " +
                                        prefvals.getDate() );
                    err = 0;
                } catch ( IOException e ) {
                    System.out.println( e.toString() );
                }
            }

        } else if ( pgmOption == WRITE ) {  

            // Create the entry if it doesn't exist
            createEntry( ld );

            // Serialize the object
            try {       
                byte[] info = serializeObject( prefvals );

                System.out.println( "Object serialized... " +
                                    "Size = " + info.length +
                                    " bytes" );

                // Write to directory
                err = writeObject( ld, PREFS_DN,
                                   "thePrefs", info );
                if ( err != 0 ) {
                    System.out.println( "Error writing to " +
                                        PREFS_DN + ": " +
                                        err );
                } else {
                    System.out.println( "Object stored at " +
                                        PREFS_DN );
                }
            } catch ( IOException e ) {
                System.out.println( e.toString() );
            }
        }
        
        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
            try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }
        System.exit( err );
    }

    /**
     * Create the entry if it doesn't exist
     *
     * @param ld Active connection to the directory
     */
    protected static void createEntry( LDAPConnection ld ) {
        try {
            // Construct the attributes of the entry
            LDAPAttributeSet attrs = new LDAPAttributeSet();
            attrs.add( new LDAPAttribute(
                "objectclass", PREFS_OC ) );
            attrs.add( new LDAPAttribute(
                "javaClassName", "Prefs") );
            byte[] info = serializeObject( new Prefs() );
            attrs.add( new LDAPAttribute(
                "javaSerializedData", info ) );
            attrs.add( new LDAPAttribute(
                "cn", "savePrefs") );
            // Construct an entry
            LDAPEntry entry = new LDAPEntry( PREFS_DN, attrs );
            // Add the entry to the directory
            ld.add( entry );
        } catch ( LDAPException e ) {
            if ( e.getLDAPResultCode() !=
                 e.ENTRY_ALREADY_EXISTS ) {
                System.out.println( e.toString() );
            }
        } catch ( IOException ioe ) {
            System.out.println( ioe.toString() );
        }
    }

    /**
     * Read the first value of a single attribute from the
     * directory and return it as a byte array
     *
     * @param ld Active connection to directory
     * @param dn DN of entry to read
     * @param attr Name of attribute to read
     * @return Value of the attribute as a byte array
     */
    public static byte[] readBytes( LDAPConnection ld,
                                    String dn,
                                    String attrName ) {
        byte[] attrVal = null;
        try {
            // Get the attribute from the directory
            LDAPEntry entry =
                ld.read( dn, new String[] { attrName } );
            LDAPAttribute attr =
                entry.getAttribute( attrName );
            // Get its first value, if there's more than one
            Enumeration enumVals = attr.getByteValues();
            if ( enumVals.hasMoreElements() ) {
                attrVal = (byte[])enumVals.nextElement();
            }
        } catch ( LDAPReferralException e ) {
            // Ignore referrals
        } catch ( LDAPException e ) {
            System.out.println( e.toString() );
        }           
        return attrVal;
    }

    /**
     * Write the serialized form of an object to the directory
     * as a byte array
     *
     * @param ld Active connection to directory
     * @param dn DN of entry to write
     * @param className Class of serialized object
     * @param data Serialized object
     * @return 0 on success, 1 on error
     */
    public static int writeObject( LDAPConnection ld,
                                   String dn,
                                   String className,
                                   byte[] data ) {

        // Use an LDAPModification object to replace the
        // current values of the entry. If they do not
        // exist, they will be created.
        LDAPModificationSet mods = new LDAPModificationSet();
        mods.add( LDAPModification.REPLACE,
                  new LDAPAttribute( ATTR_CLASS,
                                     "Prefs" ) );
        mods.add(LDAPModification.REPLACE,
                 new LDAPAttribute( ATTR_DATA,
                                    data ) );

        try {
            // Write the entry to the directory
            ld.modify( dn, mods );
        } catch ( LDAPException e ) {
            System.out.println( e.toString() );
            return 1;
        }
        return 0;
    }

    /**
     * Instantiate an object from its serialized form
     *
     * @param byteBuf Serialized form of object
     * @return Instantiated object
     * @exception IOException on failure
     */
    protected static Object deserializeObject( byte[] byteBuf )
        throws IOException {
        
        ByteArrayInputStream bis = null;
        ObjectInputStream  objis = null;

        try {
            bis = new ByteArrayInputStream( byteBuf );
            objis = new ObjectInputStream( bis );
            return objis.readObject();
        } catch( Exception ex ) {
            throw( new IOException(
                "Failed to deserialize object" ) );
        } finally {
            if ( objis != null ) {
                objis.close();
            }
            if ( bis != null ) {
                bis.close();
            }
        }
    }

    /**
     * Convert an object to its serialized form
     *
     * @param obj Object to be serialized
     * @return Serialized form of object
     * @exception IOException on failure
     */
    protected static byte[] serializeObject( Object obj )
        throws IOException {

        ByteArrayOutputStream bos = null;
        ObjectOutputStream objos = null;

        try {
            bos = new ByteArrayOutputStream();
            objos = new ObjectOutputStream(bos);
            objos.writeObject(obj);
            objos.flush();
            return bos.toByteArray();
        } catch(Exception ex) {
            throw( new IOException(
                "Failed to serialize object" ) );
        } finally {
            if (objos != null) {
                objos.close();
            }
            if (bos != null) {
                bos.close();
            }
        }
    }   
}

/**
 * Trivial class to illustrate serialization and
 * deserialization
 */
class Prefs implements Serializable {

    /**
     * Default constructor for deserialization
     */
    public Prefs() {
        this( 0, 0 ); // Default values
    }

    /**
     * Constructor that initializes with values
     *
     * @param a An integer value
     * @param b Another integer value
     */
    public Prefs(int a, int b) {
        this.a = a;
        this.b = b;
        this.d = new Date();
    }

    /**
     * Return a
     *
     * @return a
     */
    public int geta() { return a; }

    /**
     * Return b
     *
     * @return b
     */
    public int getb() { return b; }

    /**
     * Return date of construction
     *
     * @return Date of construction
     */
    public Date getDate() { return d; }

    /**
     * Assigns a
     *
     * @param a1 Value for a
     */
    public void seta(int a1) { a = a1; }

    /**
     * Assigns b
     *
     * @param b1 Value for b
     */
    public void setb(int b1) { b = b1; }

    /**
     * Assigns the construction date
     *
     * @param newDate Value for date
     */
    public void setDate(Date newDate) { d = newDate; }

    private int a;
    private int b;
    private Date d = new Date();
}


